/**
 * \file gre/lrsketch.h
 * \brief GRE::LAYER_SKETCH definition
 *
 * \if NODOC
 * $Id: lrsketch.h_v 1.6 2013/02/15 20:47:19 mju Exp $
 *
 * $Log: lrsketch.h_v $
 * Revision 1.6  2013/02/15 20:47:19  mju
 * Fix dox hdr.
 *
 * Revision 1.5  2013/01/03 22:43:24  mju
 * Add dlgExport and exportFile.
 *
 * Revision 1.4  2011/09/02 17:28:21  mju
 * Add dlgSave.
 *
 * Revision 1.3  2011/08/17 20:49:23  mju
 * Add forceCloseAll/Object, setupSketch, v_closeObject.
 *
 * Revision 1.2  2006/09/19 12:39:20  mju
 * Use rvc/object hdr.
 *
 * Revision 1.1  2006/09/18 14:16:38  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRSKETCH_H
#define	INC_GRE_LRSKETCH_H

#ifndef INC_GRE_LRCAD_H
#include <gre/lrcad.h>
#endif

#ifndef INC_RVC_OBJECT_H
#include <rvc/object.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class CAD;
	class CADELEMENT;
	class CADSTDATT;
	class STYLE;
	}

class DBEDIT;
#endif

namespace GRE {

//===================================================================================================================

//! Sketch layer.
class LAYER_SKETCH : public LAYER_CAD {
	public:

		enum WARNOUTSIDE {
			WARNOUTSIDE_Always,
			WARNOUTSIDE_Once,
			WARNOUTSIDE_Never
			};

		enum SETUPSKETCH {
			SETUPSKETCH_AllowNone,
			SETUPSKETCH_Create,
			SETUPSKETCH_Open
			};

		//! Force close of all sketch layers in specified "viewable".
		static void ForceCloseAll (
			GRE::VIEWABLE *viewable
			);

		//! Register this layer type
		static void RegisterType ();

		//! Setup sketch layer in viewable if needed and none already exists.
		static LAYER_SKETCH* SetupSketch (
			GRE::VIEW *view,							//!< View, will use active group if viewable is layout
			LAYER_SKETCH *CurSketchLayer,			//!< Current sketch layer, NULL if none
			SETUPSKETCH mode,							//!< Mode
			MDLGPARENT dlgparent = 0				//!< Parent for dialog when SETUPSKETCH_Open specified
			);

		//! Construct layer and insert into list
		explicit LAYER_SKETCH (
			GRE::GROUP *group,									//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0									//!< Reference layer for insertion
			);

		virtual ~LAYER_SKETCH ();

		//! Add element to sketch object.
		ERRVALUE AddElement (
			MDLGPARENT dlgparent,
			RVC::CADELEMENT& Element,
			const MULTISTYLE& style,
			GRE::VIEW *view
			);

		//! Delete last element in sketch (CAD) object.
		ERRVALUE DeleteLastElement ();

		//! Display dialog allowing user to define attribute table for sketch object.
		ERRVALUE DlgDefineAttribTable (
			MDLGPARENT dlgparent							//!< Parent for dialog
			);

		//! Display dialog allowing user to export sketch to file.
		ERRVALUE DlgExport (
			MDLGPARENT dlgparent,						//!< Parent for dialog
			const char *extns = 0						//!< File extensions for types allowed, 0 for all supported
			);

		//! Display dialog allowing user to save sketch object.
		ERRVALUE DlgSave (
			MDLGPARENT dlgparent							//!< Parent for dialog
			);

		//! Export sketch to specified file.
		ERRVALUE ExportFile (
			const FILEPATH& filepath					//!< Path to file to be exported, format determined from extension
			);

		//! Locate existing attribute table if any.
		ERRVALUE FindAttribTable ();

		//! Force close of sketch layer object.
		void ForceCloseObject ();

		//! Get whether to prompt user for attribute record when adding element.
		bool GetPromptForAttrib () const
			{ return (m_PromptForAttrib && HasAttribTable()); }

		//! Locate existing attribute table if any.
		RVC::STYLE& GetStyleObject (
			) { return (*m_pObjStyle); }

		//! Determine if layer has attribute table defined.
		bool HasAttribTable () const;

		//! Determine if sketch is in temporary location.
		bool IsTemporary () const
			{ return (m_IsTemporary); }

		//! Set whether to prompt user for attribute record when adding element.
		void SetPromptForAttrib (
			bool PromptForAttrib
			) { m_PromptForAttrib = PromptForAttrib; }

		//! Create or open style object used by sketch layer.
		ERRVALUE SetupStyleObject ();

		//! Set option for warning user of adding elements outside group.
		void SetWarnOutside (
			WARNOUTSIDE WarnOutside
			) { m_WarnOutside = WarnOutside; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class EXPORTER_KML;

		WARNOUTSIDE m_WarnOutside;
		bool m_IsTemporary;
		bool m_WarnOutsideDone;
		bool m_PromptForAttrib;
		int m_AttribTableNum;
		DBEDIT *m_pDbEdit;
		RVC::CAD *m_pObjSketch;
		RVC::CADSTDATT *m_pStdAttSketch;
		RVC::STYLE *m_pObjStyle;

		ERRVALUE ExportKML (const FILEPATH& filepath) const;
		ERRVALUE OpenPrivateObject (RVC::OBJECT::OPENMODE mode);

		// Overrides from LAYER
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual void v_CloseObject ();
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const char* v_SerialGetTagName () const;

		LAYER_SKETCH (const LAYER_SKETCH&);
		LAYER_SKETCH& operator= (const LAYER_SKETCH&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRSKETCH_H
