/**
 * \file mgui/ctrlmagnifier.h
 * \brief MGUI magnifier control definitions.
 *
 * \if NODOC
 * $Id: ctrlmagnifier.h_v 1.7 2011/09/01 19:39:06 mju Exp $
 *
 * $Log: ctrlmagnifier.h_v $
 * Revision 1.7  2011/09/01 19:39:06  mju
 * No legacy macro.
 *
 * Revision 1.6  2010/03/10 22:05:58  bcolombe
 * Added SetShowCenter
 *
 * Revision 1.5  2010/02/26 20:00:59  bcolombe
 * *** empty log message ***
 *
 * Revision 1.4  2010/02/26 01:35:51  bcolombe
 * Added controls, box around center pixel
 *
 * Revision 1.3  2010/02/25 17:02:59  bcolombe
 * Added srollwheel support, zoom limits
 *
 * Revision 1.2  2010/02/25 16:37:28  bcolombe
 * Added additional draw methods
 *
 * Revision 1.1  2010/02/25 00:14:47  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_CTRL_MAGNIFIER_H
#define	INC_MGUI_CTRL_MAGNIFIER_H

#ifndef INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif

#ifndef INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGE_H
#include <spatmod/imagestage.h>
#endif

#ifndef INC_SPATMOD_IMAGESTAGEAUTOPTR_H
#include <spatmod/imagestageautoptr.h>
#endif

namespace MGUI {

//===================================================================================================================
//!	Magnifier view control
class CTRL_MAGNIFIER : public CANVAS {
	public:

		enum STYLE {
			STYLE_Default =						0x00,
			STYLE_NoShowCenterCell =			0x01,
			STYLE_NoZoomControls =				0x02,
			STYLE_NoZoomFactor =				0x04
			};

		typedef fastdelegate::FastDelegate<void(double)> DELEGATE_ONZOOM;

		//! Constructor.
		CTRL_MAGNIFIER (
			);

		//! Destructor.
		virtual ~CTRL_MAGNIFIER (
			);

		//! Create canvas control
		void Create (
			LAYOUT_PANE_BASE& ParentPane,							//!< Parent pane
			INT32 Width,											//!< Desired width in pixels
			INT32 Height,											//!< Desired height in pixels
			double MinimumZoomAllowed = 2,							//!< Minimum zoom allowed
			double MaximumZoomAllowed = 16,							//!< Maximum zoom allowed
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_Expand,	//!< Sizing and alignment options
			CTRL_MAGNIFIER::STYLE style = STYLE_Default				//!< Style
			);

		//! Draw magnifier
		ERRVALUE Draw (
			const MGUI::POINT& point,				//!< Point in device coords at where to draw magnifier
			MGD::DEVICE *Device						//!< Device to draw from
			);

		//! Draw magnifier
		ERRVALUE Draw (
			const MGUI::POINT& point,				//!< Point in device coords at where to draw magnifier
			SPATMOD::IMAGE::STAGE* PtrStage,		//!< Stage to draw from
			bool TakeOwnership = true				//!< Take owernship of stage and delete
			);

		//! Draw magnifier
		ERRVALUE Draw (
			SPATMOD::IMAGE::STAGE* PtrStage,		//!< Stage to draw from, must match size of magnifier canvas
			bool TakeOwnership = true				//!< Take owernship of stage and delete
			);

		//! Erase with background color
		void Erase ();

		//! Get current zoom
		double GetZoom () const {return (m_zoom); }

		//! Redraw
		void Redraw ();

		//! Set background color of magnifier
		void SetBackgroundColor (
			const COLOR& color
			);

		//! Set delegate to call when magnifier is zoomed
		void SetDelegateOnZoom (
			DELEGATE_ONZOOM delegate
			) { m_DelegateOnZoom = delegate; }

		//! Set zoom
		void SetZoom (
			double zoom
			);

		//! Set zoom limits
		void SetZoomLimits (
			double MinimumZoomAllowed,
			double MaximumZoomAllowed
			);

		//! Set whether border around center pixel is shown
		void SetShowCenter (
			bool bShowCenter
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		double m_zoom;
		double m_MinimumZoomAllowed;
		double m_MaximumZoomAllowed;
		COLOR m_BGColor;
		COLOR m_BorderColor;
		INT32 m_style;

		LRECT2D m_ZoomInRect;
		LRECT2D m_ZoomOutRect;
		LRECT2D m_ZoomTextRect;

		SPATMOD::IMAGE::STAGE_AUTOPTR m_PrevStageAutoPtr;

		void Erase(int width, int height);
		bool OnClick (MGUI::POINT point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE state);
		void OnScrollWheel(INT32 WheelValue);

		DELEGATE_ONZOOM m_DelegateOnZoom;

		CTRL_MAGNIFIER (const CTRL_MAGNIFIER&);
		CTRL_MAGNIFIER& operator= (const CTRL_MAGNIFIER&);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MGUI::CTRL_MAGNIFIER::STYLE)
#endif

//===================================================================================================================

}	// End namespace MGUI


#endif	// INC_MGUI_CTRL_MAGNIFIER_H
