/**
 * \file mgui/formtabbed.h
 * \brief MGUI::FORM_TABBED definition
 *
 * \if NODOC
 * $Id: formtabbed.h_v 1.1 2010/03/24 20:45:48 mju Exp $
 *
 * $Log: formtabbed.h_v $
 * Revision 1.1  2010/03/24 20:45:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMTABBED_H
#define	INC_MGUI_FORMTABBED_H

#ifndef INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

namespace MGUI {

#ifndef GENERATING_DOXYGEN_OUTPUT
class ICON;
#endif

//=====================================================================================================================

//! Form with tabs to allow user to select page from one or more available pages.
class FORM_TABBED : public FORM_COMPOSITE {
	public:

		//! Form style enumeration.
		enum FORMSTYLE {
			FORMSTYLE_TabTopLeft =		0x00,		//!< Tabs on top at left (default if no other tab location specified)
			FORMSTYLE_TabTopRight =		0x01,		//!< Tabs on top at right
			FORMSTYLE_TabBottomLeft =	0x02,		//!< Tabs on bottom at left
			FORMSTYLE_TabBottomRight =	0x03,		//!< Tabs on bottom at right
			FORMSTYLE_TabLeftTop =		0x04,		//!< Tabs on left at top
			FORMSTYLE_TabLeftBottom =	0x05,		//!< Tabs on left at bottom
			FORMSTYLE_TabRightTop =		0x06,		//!< Tabs on right at top
			FORMSTYLE_TabRightBottom =	0x07,		//!< Tabs on right at bottom
			FORMSTYLE_UseIcons =			0x08,		//!< Use icons on tabs
		#ifndef GENERATING_DOXYGEN_OUTPUT
			FORMSTYLE_TabMask =			0x07,
		#endif
			};

		//! Style for pages.
		enum PAGESTYLE {
			PAGESTYLE_Default =					0,
			PAGESTYLE_IgnoreNonVisible =		0x01,		//!< Ignore non-visible items for layout of page
			PAGESTYLE_Disabled =					0x02,		//!< Page is initially disabled
			PAGESTYLE_ShowTextWithIcon =		0x04,		//!< Show both text label and icon
			};

		//! Constructor.
		FORM_TABBED ();

		//! Destructor.
		virtual ~FORM_TABBED ();

		//! Add page with text label tab to form, must be called after Create().
		//! @return Index of page, use GetPagePane() to access pane for creating controls on page.
		int AddPage (
			const MISTRING& label,						//!< Label string
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			PAGESTYLE pagestyle = PAGESTYLE_Default,
			int ChildSpacing = 4,
			int ExtraBorder = 4
			);

		//! Add page with icon tab to form, must be called after Create().
		//! @return Index of page, use GetPagePane() to access pane for creating controls on page.
		int AddPage (
			const MGUI::ICON& icon,						//!< Icon for tab
			const MISTRING& tooltip,					//!< ToolTip string
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			PAGESTYLE pagestyle = PAGESTYLE_Default,
			int ChildSpacing = 4,
			int ExtraBorder = 4
			);

		//! Create the form, must be called before AddPage().
		void Create (
			MGUI::LAYOUT_PANE_BASE& PaneParent,
			FORMSTYLE formstyle = FORMSTYLE_TabTopLeft,
			LAYOUT_SIZEALIGN sizealign = LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get index to 'active' page.
		//! @return Zero-based index to active page, -1 if none created.
		int GetActivePage (
			) const;

		//! Get reference to pane for specified page index.
		LAYOUT_PANE_BASE& GetPagePane (
			int pageindex									//!< Zero-based page index
			);

		//! Get number of tab pages.
		int GetNumPages (
			) const;

		//! Set active page by index.
		void SetActivePage (
			int pageindex									//!< Zero-based page index
			);

		//! Set delegate to call after page is activated.
		//! Note that if both page-specific and "overall" delegates are set then both will be called.
		//! Also note that delegate may not attempt to call SetActivePage(), attempts to do so will be ignored.
		void SetDelegateOnPageActivate (
			DELEGATE_VOID_NOPARMS delegate,			//!< Delegate to call
			int pageindex = -1							//!< Page to set for, -1 to set in addition to any page-specific delegate
			);

		//! Set delegate to call before page is deactivated.
		//! Note that if both page-specific and "overall" delegates are set then both will be called.
		//! Also note that delegate may not attempt to call SetActivePage(), attempts to do so will be ignored.
		void SetDelegateOnPageDeactivate (
			DELEGATE_BOOL_NOPARMS delegate,			//!< Delegate to call, return 'true' to accept, 'false' to keep current page
			int pageindex = -1							//!< Page to set for, -1 to set in addition to any page-specific delegate
			);

		//! Set whether page is enabled.
		void SetPageEnabled (
			int pageindex,									//!< Zero-based page index
			bool enabled = true
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_TABBED (const FORM_TABBED&);
		FORM_TABBED& operator= (const FORM_TABBED&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_TABBED::PAGESTYLE)
#endif

//=====================================================================================================================
}	// End namespace MGUI

#endif	// INC_MGUI_FORMTABBED_H
