/**
 * \file mgui/toolrectangle.h
 * \brief MGUI::TOOL_RECTANGLE class definitions
 *
 * \if NODOC
 * $Id: toolrectangle.h_v 1.23 2013/04/02 18:00:39 mju Exp $
 *
 * $Log: toolrectangle.h_v $
 * Revision 1.23  2013/04/02 18:00:39  mju
 * Add stepmode enum and use in setStepByToolDimension.
 *
 * Revision 1.22  2012/06/11 20:01:03  bcolombe
 * Support limiting by size
 *
 * Revision 1.21  2011/12/09 20:23:32  mju
 * No context in v_draw.
 *
 * Revision 1.20  2010/09/08 17:46:16  mju
 * Add startmode_CenterlineWidth and enum for adjustMode.
 *
 * Revision 1.19  2010/06/24 15:10:52  mju
 * Add mode_RotateSide01/23.
 *
 * Revision 1.18  2010/06/23 20:09:21  mju
 * Add option to show handles for rotation around opposite side.
 * Add startmode settings.
 *
 * Revision 1.17  2009/09/15 20:58:26  mju
 * Add keystate parm to adjustTool.
 *
 * Revision 1.16  2009/08/26 21:57:44  mju
 * Add MoveByStep.
 *
 * Revision 1.15  2009/07/23 22:10:12  bcolombe
 * Added support for stepping by % of profile tool size
 *
 * Revision 1.14  2008/07/22 14:47:49  bcolombe
 * Removed tooltype, added method to set stepping by tool dimension
 *
 * Revision 1.13  2008/07/15 15:24:31  bcolombe
 * Added tooltype and SetAngelVisible now can extend rotation handle outside box
 * for pointprofile tool
 *
 * Revision 1.12  2008/07/08 20:15:30  scowan
 * Added manual entry form.
 *
 * Revision 1.10  2008/02/01 18:04:29  mju
 * Add setSideMarkerLength.
 *
 * Revision 1.8  2008/01/31 15:56:25  mju
 * Add zone_setcenter.
 *
 * Revision 1.6  2008/01/03 18:32:51  mju
 * Add setAngle/CenterVisible.
 *
 * Revision 1.5  2007/12/17 16:14:54  mju
 * Change op= to setRectangle and add setCorners.
 *
 * Revision 1.1  2003/01/10 23:08:50  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLRECTANGLE_H
#define	INC_MGUI_TOOLRECTANGLE_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

class POLYLINE;

namespace MGUI {


//! Rectangle tool class.
class TOOL_RECTANGLE : public TOOL_LIMITED {
	public:

		enum DIRECTION {
			DIRECTION_Up =			0,
			DIRECTION_Forward =	DIRECTION_Up,
			DIRECTION_Down =		1,
			DIRECTION_Backward =	DIRECTION_Down,
			DIRECTION_Left =		2,
			DIRECTION_Right =		3
			};

		enum STARTMODE {
			STARTMODE_Default =				0,		//!< Upright rectangle, start at one corner, drag opposite corner
			STARTMODE_CenterlineWidth =	1,		//!< Press-drag-release to define centerline, click again to define width
			};

		enum STEPMODE {
			STEPMODE_None =				0,		//!< No stepping by key
			STEPMODE_Screen =				1,		//!< Step in "screen" orientation
			STEPMODE_ByOrientation =	2,		//!< Up arrow moves in direction of orientation angle (0 = right)
			};

		//! Constructor.
		explicit TOOL_RECTANGLE (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_RECTANGLE ();

		//! Get rotation angle.
		//! @return Rotation angle in RADIANS.
		double GetAngle (
			) const { return (m_angle); }

		//! Get center of rectangle in tool coordinates.
		DPOINT2D GetCenter (
			) const { return (m_extents.GetCenter()); }

		//! Get coordinates of specified corner.
		const DPOINT2D& GetCorner (
			int corner									//!< Corner number (0-3)
			) const { return (m_corners[corner]); }

		//! Get coordinates of all rectangle corners.
		//! @return Array of 4 corner coordinates.
		const DPOINT2D* GetCorners (
			) const { return (m_corners); }

		//! Copy corner coordinates into specified array.
		void GetCorners (
			DPOINT2D rcorners[4]						//!< Array to contain corner coordinates
			) const { for (int i = 0; (i < 4); ++i) rcorners[i] = m_corners[i]; return; }

		//! Get extents of tool.
		const DRECT2D& GetExtents (
			) const { return (m_extents); }

		//! Get current rectangle size.
		//! @return Size in tool coordinates before rotation.
		const DPOINT2D& GetSize (
			) const { return (m_size); }

		//! Move tool one "step" in specified direction.
		void MoveByStep (
			DIRECTION direction,							//!< Direction
			double StepAmtOverride = 0					//!< Step amount override, 0 to use value set in tool
			);

		//! Set whether rotation angle indicator is visible.
		//! The rotation angle is indicated by a line from the center to the boundary.
		void SetAngleVisible (
			bool anglevisible = true,
			bool sideanglevisible = false,
			int pixels = 0	//!< Number of pixels to extend handle outside of box
			);

		//! Set aspect ratio for rectangle.
		//! Setting the aspect ratio to a value <= 0 removes the aspect constraint.
		void SetAspect (
			double aspect									//!< Aspect ratio X/Y
			);

		//! Set whether center marker is visible.
		void SetCenterVisible (
			bool centervisible = true
			);

		//! Set corners from POLYLINE.
		//! Polyline must contain 4 vertices (5 if closed).
		void SetCorners (
			const POLYLINE& corners						//!< Corners in tool coordinates.
			);

		//! Set default angle for starting new rectangle.
		void SetDftAngle (
			double dftangle								//!< Default angle in degrees
			);

		//! Set position from rectangle.
		void SetRectangle (
			const DRECT2D& rhs							//!< Rectangle in tool coordinates to set
			);

		//! Set length of "side markers".
		//! By default, no side markers are shown.
		//! This is useful if box indicates a target if it can become small and hard to see.
		void SetSideMarkerLength (
			int lengthoutside								//!< Length of marker outside box in pixels
			);

		void SetSizeLimits (
			double MinWidth,
			double MaxWidth,
			double MinHeight,
			double MaxHeight
			);

		void SetSizeLimits (
			double MaxArea
			);

		//! Set mode for when user starts new box.
		void SetStartMode (
			STARTMODE StartMode							//! Mode for when user starts new box
			) { m_StartMode = StartMode; }

		//! Set to step by tool dimension, supported in v_OnKey
		void SetStepByToolDimension (
			double StepAmt = 1.0,						//!< Size relative to width/height of profile tool (1.0 = 100%)
			STEPMODE StepMode = STEPMODE_Screen
			) { m_StepAmt = StepAmt; m_StepMode = static_cast<UINT8>(StepMode); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Enumeration constants
		enum ZONE {
			ZONE_Outside = 0,
			ZONE_Inside,
			ZONE_Corner0,
			ZONE_Corner1,
			ZONE_Corner2,
			ZONE_Corner3,
			ZONE_Side01,
			ZONE_Side12,
			ZONE_Side23,
			ZONE_Side30,
			ZONE_Rotate,
			ZONE_RotateLeft,
			ZONE_RotateRight,
			ZONE_RotateSide01,
			ZONE_RotateSide23,
			ZONE_SetCenter,
			ZONE_NoFunction
			};
		enum ADJUSTMODE {
			ADJUSTMODE_Normal,
			ADJUSTMODE_Rotate,
			ADJUSTMODE_Create2					// Second step of creation where have to click again
			};

		// Tool settings
		double m_aspect;							// Aspect ratio, proportional if > 0
		bool m_centervisible;					// Visibility of center indicator
		bool m_anglevisible;						// Visibility of rotation angle indicator
		bool m_sideanglevisible;
		UINT8 m_StepMode;
		int m_AngleHandlePixels;
		int m_sidemarkerlength;					// Length in pixels of side markers out of box
		STARTMODE m_StartMode;
		double m_StartSize;
		// Tool position values
		DRECT2D m_extents;						// Position (extents if rotated)
		DPOINT2D m_corners[4];					// Corner coordinates
		DPOINT2D m_size;							// Current size
		double m_dftangle;
		double m_angle;							// Rotation angle
		double m_StepAmt;							// Step size percentage
		double m_MinHeight;
		double m_MaxHeight;
		double m_MinWidth;
		double m_MaxWidth;
		double m_MaxArea;
		// Values used during interaction
		ZONE m_PointerZone;						// Current cursor zone
		DPOINT2D m_prevpt;						// Previous point during button-down
		ADJUSTMODE m_AdjustMode;				// Adjustment mode

		//! Manual Rectangle Entry form forward declaration
		class MANUAL_RECTANGLE;
		friend class MANUAL_RECTANGLE;
		
		// Methods
		void AdjustTool (const DPOINT2D& pt, KEYSTATE keystate, bool firsttime=false);
		DPOINT2D ComputeCenter () const;
		void ComputePointerZone (const DPOINT2D&);
		void Recompute ();
		void ResizeTool (const DPOINT2D&);
		void RotateTool (const DPOINT2D&);
		void StartTool (const DPOINT2D&);
		void UpdateCursor (const DPOINT2D&) const;

		// TOOL_BASE overrides.
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual bool v_OnKey (MGUI::POINT devpt, KEYCODE keycode, KEYSTATE keystate);
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual bool v_OnPointerButtonUp (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerLeave ();
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		// TOOL_LIMITED overrides.
		virtual void v_OnChangeLimits ();
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


}	// End namespace MGUI


#endif	// INC_MGUI_TOOLRECTANGLE_H
