/*****************************************************************************
 *
 * \file mi32/htmldom.h
 * \brief HTML DOC and support classes
 *
 * $Log: htmldoc.h_v $
 * Revision 1.33  2008/09/09 21:48:57  mju
 * More doxygen issues.
 *
 * Revision 1.32  2008/09/08 13:14:57  mju
 * Fix doxygen warning.
 *
 * Revision 1.31  2007/08/29 15:42:39  dwilliss
 * *** empty log message ***
 *
 * Revision 1.30  2007/08/28 18:09:54  dwilliss
 * Fixed table border problems. May still be off on height.
 *
 * Revision 1.29  2007/08/28 16:35:58  dwilliss
 * Added an elem num member to elements so we can keep them in defined order
 * while sorting by z-order
 *
 * Revision 1.28  2007/08/22 16:42:03  dwilliss
 * added style method to check to see if it's a "special state" (hover, etc)
 *
 * Revision 1.27  2007/08/21 16:32:48  dwilliss
 * Added private version of SetState to control propegation.  Public version
 * calls private version.
 *
 * Revision 1.26  2007/07/26 21:27:01  dwilliss
 * Added a style member for tab stop
 *
 * Revision 1.25  2007/07/25 22:50:31  dwilliss
 * Cursor doesn't exist for non-MGUI
 *
 * Revision 1.24  2007/07/25 20:23:47  dwilliss
 * cursor id is available under MFC now
 *
 * Revision 1.23  2007/07/25 14:52:28  dwilliss
 * added a member and method to element to do more during layout and less
 * during redraw
 *
 * Revision 1.22  2007/07/18 17:02:33  dwilliss
 * added relative to enums for self
 *
 * Revision 1.21  2007/07/12 21:55:40  dwilliss
 * added stuff for z-index support
 *
 * Revision 1.20  2007/07/12 20:23:22  dwilliss
 * a few private API changes
 *
 * Revision 1.19  2007/07/09 16:09:55  dwilliss
 * added a valign enum and valign numeric value
 *
 * Revision 1.18  2007/07/06 19:55:08  dwilliss
 * *** empty log message ***
 *
 * Revision 1.17  2007/07/06 14:57:28  dwilliss
 * Implemented CSS text-transform
 *
 * Revision 1.16  2007/06/08 16:32:52  scowan
 * There is not mgui cursor for win32 native code.
 *
 * Revision 1.15  2007/06/07 22:28:28  dwilliss
 * Lots of changes for pseudo-classes
 *
 * Revision 1.14  2007/06/01 20:55:26  dwilliss
 * added optional position and clipping parameters to draw methods
 *
 * Revision 1.13  2007/05/24 17:16:29  dwilliss
 * methods to combine html fragments and convert {~} codes to html.
 *
 * Revision 1.12  2007/05/18 22:51:56  dwilliss
 * added method to combine html fragments
 *
 * Revision 1.11  2007/05/18 20:47:44  dwilliss
 * Added IsHTMLFragment static method
 *
 * Revision 1.10  2007/04/12 16:03:32  dwilliss
 * changed how we handle the enums in styles to simplify implementation
 *
 * Revision 1.9  2007/04/04 19:04:37  dwilliss
 * Added method to set source file
 *
 * Revision 1.8  2007/03/30 22:35:02  dwilliss
 * added some members to element to keep track of child offset so we
 * don't have to recompute
 *
 * Revision 1.7  2007/03/23 16:33:36  dwilliss
 * more methods
 *
 * Revision 1.6  2007/03/20 21:36:29  dwilliss
 * private methods
 *
 * Revision 1.5  2007/03/14 22:55:19  dwilliss
 * some new methods
 *
 * Revision 1.4  2007/03/09 22:02:50  dwilliss
 * Started implementing affine transformation on rendering
 *
 * Revision 1.3  2007/03/08 23:27:25  dwilliss
 * Added some methods
 *
 * Revision 1.2  2007/03/06 00:01:51  dwilliss
 * more working
 *
 * Revision 1.1  2007/02/27 15:56:25  dwilliss
 * Initial revision
 *
 *
 **/

#ifndef INC_MI32_HTMLDOC_H
#define INC_MI32_HTMLDOC_H

#ifndef INC_MI32_MISTRINGLIST_H
#include <mi32/mistringlist.h>
#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

struct TEXTSTYLE;
struct DPOINT2D;
struct DRECT2D;
class REGION2D;
class TRANS2D_AFFINE;
namespace MGD {
	class DEVICE;
	class CONTEXT;
	}

namespace HTML {
	class ELEMENT;

#ifndef GENERATING_DOXYGEN_OUTPUT
class STYLESHEET;
struct INLINE_POS;
#endif


class HTMLDOC : public XMLDOC {
	public:
		typedef fastdelegate::FastDelegate<void(const DRECT2D& clip)> DELEGATE_REDRAW;
	
		HTMLDOC();
		virtual ~HTMLDOC();

		//! Add an inline image that can be rendered using an \<img\> tag with
		//! \<img src='inline:tag'\>.
		//! If you bTakeOwnership is true, then the HTMLDOC will "own" the device
		//! and will delete it when it's done.  If not, the HTMLDOC will copy 
		//! the device (not terribly efficient)
		void AddInlineImage (
			const MISTRING& key,
			MGD::DEVICE* Device,
			bool bTakeOwnership
			);

		//! Compute the size of the HTML document.
		//! Call GetHeight()/GetWidth() after computing the size.
		void DoLayout (
			MGD::CONTEXT* context,	//!< MGD context -- needed for size calculations
			double MaxWidth			//!< Maximum width in pixels
			);

		//! Draws the document in the given MGD context
		void Draw (
			MGD::CONTEXT* context,
			const DPOINT2D* offset = 0,	//!< If not NULL, the coordinate of the upper-left corner of the document
			const DRECT2D* cliprect = 0,	//!< If not NULL, the portion of the document to draw
			bool bDrawAll = true			//!< If false, only draw elements marked as needing to be redrawn
			);

		//! Find an element at a given point (const).
		const ELEMENT* FindElement (
			const DPOINT2D& pt
			) const;

		//! Find an element at a given point (non-const).
		ELEMENT* FindElement (
			const DPOINT2D& pt
			);

		// Get the extents of all elements marked as "dirty"
		void GetDirtyRegion (
			REGION2D& region
			);

		//! Get the height of the document in pixels
		//! Note: you must call DoLayout() after parsing before calling
		//! GetHeight().
		//! If the affine transformation includes rotation, the height is of
		//! the unrotated content.
		double GetHeight (
			) const { return (m_Height); }

		//! Check to see if there is a hyper link at a given point.
		//! Returns true if the point is a hyper link and returns the URL
		//! in the href parameter.
		bool GetLink (
			const DPOINT2D& pt,
			MISTRING& href
			) const;


		//! Gets the contents of the \<title\> tag (if any) or the first line
		//! of text.
		//! The maxlen parameter controls how much of the body text to get
		//! if there is no title tag.
		void GetTitleOrFirstLine (
			MISTRING& string,
			int maxlen = 80
			) const;

		//! Check to see if there is a tooltip for a given point.
		//! Returns true if the point is over an element with a title
		//! attribute or an image with an alt attribute.
		//! Returns the tooltip text in the tooltip parameter.
		bool GetTooltip (
			const DPOINT2D& pt,
			MISTRING& tooltip
			) const;

		//! Get the width of the document in pixels
		//! Note: you must call DoLayout() after parsing before calling
		//! GetWidth();
		//! If the affine transformation includes rotation, the width is of
		//! the unrotated content.
		double GetWidth (
			) const { return (m_Width); }

		double GetZoom (
			) const { return m_Zoom; }

		//! Determine if the document requires accurate zooming.
		//! The default is false unless SetAccurateZoom() has been called.
		bool GetAccurateZoom (
			) const;

		//! Get the value of the bFullRedraw parameter to the last Draw call
		//! Used by elements during a redraw
		bool GetFullRedraw (
			) const;

		//! Parse an in-memory HTML document
		//! The buffer is assumed to be UTF8
		ERRVALUE Parse (
			const UINT8* buffer
			);

		//! Read and parse an HTML file
		ERRVALUE Read (
			const FILEPATH& filepath
			);

		//! Set the default text style for the body
		void SetDefaultTextStyle (
			const TEXTSTYLE& Style
			);

		//! Set an affine transformation for rendering the output.  
		//! NOTE: Currently the only transformation supported is scale, and it
		//! assumes that xscale == yscale
		//! All scaling is applied when doing the layout.  Offset and Rotation
		//! are only applied on Draw()
		//! Note: calling this method forces accurate zooming (See 
		//! SetAccurateZoom() for details)
		void SetTransAffine (
			const TRANS2D_AFFINE& trans
			);

		//! Set the source file.
		//! The source file is used if the HTML refers to other files using
		//! relative paths.
		//! This can be used if the source was read into memory and parsed
		//! using Parse() instead of Read()
		void SetSourceFile (
			const FILEPATH& url
			);

		//! Set the source URL
		//! The source URL is used if the HTML refers to other files using
		//! relative paths.
		void SetSourceURL (
			const MISTRING& url
			);
		//! Set the zoom factor of the document
		//! Default zoom is 1.0.  This forces offset and rotation to 0.
		//! Note: setting zoom forces accurate zooming (See SetAccurateZoom()
		//! for details)
		void SetZoom (
			double zoom
			);

		//! Set accurate zooming.
		//! If set, the CSS measurement 1px will be assumed to be approx 1/100th
		//! of an inch regardless of the actual device resolution.  If not set,
		//! 1px == 1 device pixel.  This must be called BEFORE DoLayout()
		void SetAccurateZoom (
			bool val = true
			);

		//! Set a delegate to call when things need to be redrawn.
		//! The delegate will be given a DRECT2D with the absolute extents of the
		//! area to be redrawn
		void SetDelegateRedraw (
			DELEGATE_REDRAW delegate
			) { m_DelegateRedraw = delegate; }

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		friend class ELEMENT;		// ELEMENT can call methods on m_Priv
		friend class BORDER;			// BORDER can call methods on m_Priv
		friend class STYLESHEET;	// STYLESHEET can call methods on m_Priv

		class PRIV;
		PRIV* m_Priv;

		double m_Height;
		double m_Width;
		double m_Zoom;
		DELEGATE_REDRAW m_DelegateRedraw;

		ERRVALUE BuildTree(XMLNODE*);
		ERRVALUE BuildTreeStyleNode(XMLNODE*);

		bool GetFirstLine (MISTRING& mistr, XMLNODE*node, int maxlen = 80) const;

#endif
	};


	//! Determine if given text is HTML.
	//! This looks for a leading DOCTYPE tag or \<html\> tag
	bool TextIsHTML (
		const MIUNICODE* string
		);

	//! Determine if given text is an HTML fragment.
	//! This looks for HTML tags within the text
	bool TextIsHTMLFragment (
		const MIUNICODE* string
		);

	//! Combines multiple HTML fragments into one HTML document.
	//! Each fragment is assumed to be a block element and will be placed into
	//! a div or a table row.  Consecutive non-HTML fragments will be combined
	//! together (joined by a newline) and converted to HTML as a group.  This
	//! will allow multiple tab-delimited lines to be converted into a single
	//! HTML table.
	//! If any of the fragments are full HTML (with a head and body), the
	//! result heads of those fragments will be returned in the pHead parameter
	//! (unless it's NULL, in which case they're just ignored)
	//! 
	//! The result is an HTML fragment without a html or body tag.
	void CombineHTMLFragments (
		MISTRING& Result,
		const MISTRINGLIST& fragments,
		MISTRING* pHead = 0
		);

	} // end of HTML namespace


#endif

