/**
 * \file <mi32/lidarclassdesc.h>
 * \brief LIDAR::CLASSDESC definitions
 *
 * \if NODOC
 * $Id: lidarclassdesc.h_v 1.9 2012/10/10 15:43:55 mju Exp $
 *
 * $Log: lidarclassdesc.h_v $
 * Revision 1.9  2012/10/10 15:43:55  mju
 * Add classdescset::getItemNameDisplay.
 *
 * Revision 1.8  2012/08/09 20:12:33  mju
 * Add getCodePredefined.
 *
 * Revision 1.7  2012/08/08 17:02:25  mju
 * Add classid.
 *
 * Revision 1.6  2012/06/19 21:30:57  mju
 * Add usage_RoadEdge.
 *
 * Revision 1.5  2012/06/15 14:45:13  mju
 * Add usage_reserved.
 *
 * Revision 1.4  2012/06/14 21:32:12  mju
 * Add classes for las 1.4.
 *
 * Revision 1.3  2011/07/20 20:27:56  mju
 * Add is/setUsed and save whether used in xml.
 *
 * Revision 1.2  2011/07/18 20:16:03  mju
 * Add includeAll in getXML.
 *
 * Revision 1.1  2011/06/24 21:10:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARCLASSDESC_H
#define	INC_MI32_LIDARCLASSDESC_H

#ifndef	INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_CODESPACE_H
#include <mi32/codespace.h>
#endif

#ifndef	INC_MI32_BOOL3_H
#include <mi32/bool3.h>
#endif


#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class CLASSDESCARRAY;
class FILEPATH;
class FILEPATHLIST;
class XMLNODE;
#endif

namespace LIDAR {

//=====================================================================================================================

//! Lidar Class description.
class CLASSLIBEXPORT CLASSDESC {
	public:

		//! Predefined usages for classes.
		enum USAGE {
			USAGE_Unspecified =					0,		//!< No predefined usage specified
			USAGE_NeverClassified =				1,
			USAGE_Unclassified =					2,
			USAGE_Ground =							3,
			USAGE_LowVegetation =				4,
			USAGE_MediumVegetation =			5,
			USAGE_HighVegetation =				6,
			USAGE_Building =						7,
			USAGE_LowNoise =						8,
			USAGE_ModelKeyPoint =				9,
			USAGE_Water =							10,
			USAGE_NonGround =						11,
			USAGE_Tower =							12,	//!< Tower, pole, support structure
			USAGE_Cable =							13,	//!< Cable, wire
			USAGE_BridgeDeck =					14,
			USAGE_RoadSurface =					15,
			USAGE_Vehicle =						16,
			USAGE_Animal =							17,
			USAGE_Object =							18,
			USAGE_Fence =							19,
			USAGE_Rail =							20,
			USAGE_Wire_Guard =					21,
			USAGE_Wire_Conductor =				22,
			USAGE_Wire_StructureConnector =	23,
			USAGE_BridgeStructure =				24,	//!< Bridge structure other than deck
			USAGE_HighNoise =						25,
			USAGE_RoadEdge =						26,

			USAGE_Predefined_Count,
			USAGE_Reserved =				253,
			USAGE_Excluded =				254,	//!< Culled or Deleted
			USAGE_Other =					255,
			};

		//! Get TEXTID for usage.
		static TEXTID GetUsageTextID (
			USAGE usage
			);

		//! Default constructor.
		CLASSDESC ();

		//! Destructor.
		~CLASSDESC ();

		//! Get color reference for generating style.
		UINT32 GetColorRef () const
			{ return (m_ColorRef); }

		//! Get whether to always include class in table.
		bool GetIncludeAlways () const
			{ return (m_IncludeAlways); }

		//! Get class name.
		const MISTRING& GetName () const
			{ return (m_Name); }

		//! Get usage.
		USAGE GetUsage () const
			{ return (m_Usage); }

		//! Determine if is system default.
		bool IsDefault () const
			{ return (m_IsDefault); }

		//! Determine if class is used.
		BOOL3 IsUsed () const
			{ return (m_IsUsed); }

		//! Set values.
		void Set (
			USAGE usage,
			const MISTRING& name,
			UINT32 ColorRef,
			bool IncludeAlways,
			bool IsDefault = false
			);

		//! Set color reference.
		void SetColorRef (
			UINT32 ColorRef
			) { m_ColorRef = ColorRef; m_IsDefault = false; }

		//! Set whether to always include.
		void SetIncludeAlways (
			bool IncludeAlways
			) { m_IncludeAlways = IncludeAlways; m_IsDefault = false; }

		//! Set class name.
		void SetName (
			const MISTRING& name
			) { m_Name = name; m_IsDefault = false; }

		//! Set class usage.
		void SetUsage (
			USAGE Usage
			) { m_Usage = Usage; m_IsDefault = false; }

		//! Set whether class is used.
		void SetUsed (
			BOOL3 used
			) { m_IsUsed = used; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		USAGE m_Usage;
		bool m_IncludeAlways;				// Always include in table
		bool m_IsDefault;
		BOOL3 m_IsUsed;
		MISTRING m_Name;
		UINT32 m_ColorRef;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Set of Lidar class descriptions.
class CLASSLIBEXPORT CLASSDESCSET {
	public:

		//! Get FILEPATHLIST for previously saved class arrays.
		static void GetSavedPathList (
			FILEPATHLIST& FilePathList			//!< Path list returned.
			);

		//! Get "shared" class array used for auto-linking and elsewhere.
		static const CLASSDESCSET& GetShared ()
			{ return (GetSharedLow()); }

		//! Read (reload) shared class array using current Ini setting.
		static ERRVALUE ReadShared ();

		//! Default constructor, initializes to system (ASPRS) default.
		CLASSDESCSET ();

		//! Copy constructor.
		CLASSDESCSET (const CLASSDESCSET& rhs);

		//! Destructor.
		~CLASSDESCSET ();

		//! Assignment.
		CLASSDESCSET& operator= (const CLASSDESCSET& rhs);

		//! Copy to "shared" class array.
		void CopyToShared (
			bool UpdateIniDefault			//!< true to update default specified in Ini file
			) const;

		//! Find first class with specified "usage".
		//! @return Class number or -1 if no class found.
		int FindUsageClass (
			CLASSDESC::USAGE usage
			) const;

		//! Get number of classes.
		UINT16 GetClassCount () const;

		//! Get codes for predefined set if used.
		//! @return Code or CODESPACE_Unknown if not using predefined set.
		CODESPACE GetCodePredefined () const;

		//! Get filepath saved to.
		const FILEPATH& GetFilePath () const;

		//! Get item for class (const).
		const CLASSDESC& GetItem (
			UINT16 ClassNumber				//!< Class number
			) const;

		//! Get item for class.
		CLASSDESC& GetItem (
			UINT16 ClassNumber				//!< Class number
			);

		//! Get string for displaying item name, will include class number.
		MISTRING GetItemNameDisplay (
			UINT16 ClassNumber,				//!< Class number
			bool IncludeNumberUndefined	//!< Include number for undefined classes
			) const;

		//! Get XMLNODE representation.
		XMLNODE* GetXML (
			XMLNODE *parentnode,
			const char *nodename = "LidarClasses",
			bool IncludeAll = false
			) const;

		//! Read array from specified file.
		ERRVALUE Read (
			const FILEPATH& filepath
			);

		//! Reset to predefined set for codespace (not same as "shared").
		void Reset (
			CODESPACE codespace
			);

		//! Set item for class.
		void SetItem (
			UINT16 ClassNumber,				//!< Class number
			const CLASSDESC& classdesc
			);

		//! Set from XML.
		//! Will clear stored filepath.
		ERRVALUE SetFromXML (
			const XMLNODE *xmlnode
			);

		//! Write array to specified file.
		//! Updates current filepath to that specified.
		ERRVALUE Write (
			const FILEPATH& filepath
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		static CLASSDESCSET& GetSharedLow ();

		#endif
	};

//=====================================================================================================================

//! Container for class identifier.
//! Contains "usage" and "class number" to allow either or both to be used.
class CLASSID {
	public:

		//! Default constructor.
		CLASSID (
			): m_usage(CLASSDESC::USAGE_Unspecified), m_number(UINT16_MAX) { }

		//! Construct with "usage" only.
		explicit CLASSID (
			CLASSDESC::USAGE usage
			): m_usage(static_cast<UINT16>(usage)), m_number(UINT16_MAX) { }

		//! Construct with "class number" only.
		explicit CLASSID (
			UINT16 ClassNumber
			): m_usage(CLASSDESC::USAGE_Unspecified), m_number(ClassNumber) { }

		//! Construct with "usage and "class number".
		CLASSID (
			CLASSDESC::USAGE usage,
			UINT16 ClassNumber
			): m_usage(static_cast<UINT16>(usage)), m_number(ClassNumber) { }

		//! Get class number.
		UINT16 GetNumber () const
			{ return (m_number); }

		//! Get usage.
		CLASSDESC::USAGE GetUsage () const
			{ return (static_cast<CLASSDESC::USAGE>(m_usage)); }

		//! Determine if has specified class number.
		bool HasNumber () const
			{ return (m_number != UINT16_MAX); }

		//! Set usage and class number.
		void Set (
			CLASSDESC::USAGE usage,
			UINT16 ClassNumber
			) { m_usage = static_cast<UINT16>(usage); m_number = ClassNumber; }

		//! Set class number.
		void SetNumber (
			UINT16 ClassNumber
			) { m_number = ClassNumber; }

		//! Set usage.
		void SetUsage (
			CLASSDESC::USAGE usage
			) { m_usage = static_cast<UINT16>(usage); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT16 m_usage;
		UINT16 m_number;
		#endif
	};

//=====================================================================================================================

}	// End namespace LIDAR

#undef CLASSLIBEXPORT

#endif	// INC_MI32_LIDARCLASSDESC_H
