/**
 * \file mi32/lidarpoint32.h
 * \brief LIDAR POINTARRAY32 and POINTRECORD32 definitions
 *
 * \if NODOC
 * $Id: lidarpoint32.h_v 1.6 2012/09/24 18:57:53 mju Exp $
 *
 * $Log: lidarpoint32.h_v $
 * Revision 1.6  2012/09/24 18:57:53  mju
 * Use int32 not uint32 for getCoordXYZ.
 *
 * Revision 1.5  2012/09/18 13:57:16  mju
 * Dox ctor fix.
 *
 * Revision 1.4  2011/07/15 18:04:15  mju
 * *** empty log message ***
 *
 * Revision 1.3  2011/07/08 13:20:05  mju
 * Use 2d/3d in calc methods.
 *
 * Revision 1.1  2011/06/24 21:10:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARPOINT32_H
#define	INC_MI32_LIDARPOINT32_H

#ifndef	INC_MI32_RECORDARRAY_H
#include <mi32/recordarray.h>
#endif

#ifndef	INC_MI32_QUANTIZER_H
#include <mi32/quantizer.h>
#endif

#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

namespace LIDAR {

//=====================================================================================================================

//! Array of point records where first 3 fields are X, Y, Z quantized to INT32.
class CLASSLIBEXPORT POINTARRAY32 : public RECORDARRAY {
	public:

		//! Constructor.
		POINTARRAY32 ();

		//! Destructor.
		~POINTARRAY32 ();

		//! Get quantizer.
		const QUANTIZER_LPOINT3D& GetQuantizer () const
			{ return (m_Quantizer); }

		//! Set quantizer.
		void SetQuantizer (
			const QUANTIZER_LPOINT3D& quantizer
			) { m_Quantizer = quantizer; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		QUANTIZER_LPOINT3D m_Quantizer;

		// Unimplemented.
		POINTARRAY32 (const POINTARRAY32&);
		POINTARRAY32& operator= (const POINTARRAY32&);
		#endif
	};

//=====================================================================================================================

//! Point record where first 3 fields are X, Y, Z as INT32 with quantization.
class CLASSLIBEXPORT POINTRECORD32 : public RECORDARRAY::RECORD {
	public:

		//! Default constructor, creates NULL record.
		POINTRECORD32 (
			) { }

		//! Copy constructor.
		POINTRECORD32 (
			const POINTRECORD32& rhs
			) : RECORD(rhs) { }

		//! Construct and attach to array.
		POINTRECORD32 (
			POINTARRAY32& Array
			) : RECORD(Array) { }

		//! Construct and attach to array with index.
		POINTRECORD32 (
			POINTARRAY32& Array,
			UINT32 ArrayIdx
			) : RECORD(Array,ArrayIdx) { }

		//! Destructor.
		~POINTRECORD32 ()
			{ }

		//! Get (translated) X coordinate.
		double GetCoordX (
			) const { return (GetQuantizer().GetX().CalcDouble(reinterpret_cast<const INT32*>(GetData())[0])); }

		//! Get (translated) Y coordinate.
		double GetCoordY (
			) const { return (GetQuantizer().GetY().CalcDouble(reinterpret_cast<const INT32*>(GetData())[1])); }

		//! Get (translated) Z coordinate.
		double GetCoordZ (
			) const { return (GetQuantizer().GetZ().CalcDouble(reinterpret_cast<const INT32*>(GetData())[2])); }

		//! Get (translated) 2D coordinates.
		DPOINT2D GetCoord2D (
			) const { return (GetQuantizer().CalcDouble2D(*reinterpret_cast<const LPOINT2D*>(GetData()))); }

		//! Get (translated) 3D coordinates.
		DPOINT3D GetCoord3D (
			) const { return (GetQuantizer().CalcDouble3D(GetCoordRaw())); }

		//! Get raw coordinates.
		const LPOINT3D& GetCoordRaw (
			) const { return (*reinterpret_cast<const LPOINT3D*>(GetData())); }

		//! Get quantizer.
		const QUANTIZER_LPOINT3D& GetQuantizer () const
			{ return (static_cast<const POINTARRAY32&>(GetArray()).GetQuantizer()); }

		//! Set point 2D coordinates.
		void SetCoord2D (
			const DPOINT2D& pt
			) { GetQuantizer().CalcInt2D(pt,*reinterpret_cast<LPOINT2D*>(GetData())); }

		//! Set point 3D coordinates.
		void SetCoord3D (
			const DPOINT3D& pt
			) { GetQuantizer().CalcInt3D(pt,*reinterpret_cast<LPOINT3D*>(GetData())); }

		//! Set point Z coordinate.
		void SetCoordZ (
			double z
			) { reinterpret_cast<INT32*>(GetData())[2] = GetQuantizer().GetZ().CalcInt(z); }

	};

//=====================================================================================================================

}	// End namespace LIDAR

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_LIDARPOINT32_H
