/******************************************************************************
 * \file mi32/mg2rastc.h
 * \brief A base colormapped raster-based MGD::DEVICE.  
 *
 * \if NODOC
 * $Id: mg2rastc.h_v 1.10 2011/11/02 14:01:55 mju Exp $
 *
 * $Log: mg2rastc.h_v $
 * Revision 1.10  2011/11/02 14:01:55  mju
 * Remove redundant hdr.
 *
 * Revision 1.9  2007/01/19 16:09:48  mju
 * Use color_array.
 *
 * Revision 1.8  2006/06/27 20:15:33  scowan
 * Changed create method signature to take a non-const objitem.
 *
 * Revision 1.7  2005/01/26 18:46:19  mju
 * Move colormap member here and add virtual method to get.
 *
 * Revision 1.2  2003/05/19 20:50:22  dwilliss
 * Added COLORPOLICY
 *
 * Revision 1.1  2003/05/19 17:48:58  dwilliss
 * Initial revision
 * \endif
 */

#ifndef INC_MI32_MG2RASTC_H
#define INC_MI32_MG2RASTC_H

#ifndef INC_MD2RAST8_H
#include <mi32/mg2rast8.h>
#endif

namespace MGD {


class CONTEXT_RASTER_COLORMAPPED;

class DEVICE_RASTER_COLORMAPPED: public DEVICE_RASTER_GRAY8 {

	public:

		//! How to deal with SetColor() if the color isn't in the colormap
		enum COLORPOLICY {
			COLORPOLICY_UseClosest = 0,		//!< Use the closest matching color
			COLORPOLICY_AddColor = 1,			//!< Add the color to the colormap (falls back to UseClosest once colormap is full)
			COLORPOLICY_Transparent = 2,		//!< Treat it as a transparent color
			COLORPOLICY_Fallback = 3,			//!< Fallback to value set via SetFallbackPixel() (default = 0)
			};

		//! Construct a colormapped raster device.
		//! After constructing, you should call Create()
		DEVICE_RASTER_COLORMAPPED();

		//! Destructor
		virtual ~DEVICE_RASTER_COLORMAPPED();

		// (Note: copy constructor private and unimplemented.  Can't copy one.)


		//! Create a raster to draw into
		//! To draw into an existing raster, call Open
		ERRVALUE Create (
			RVC::OBJITEM& ObjItem,
			RVC::RASTER::MAKEPARMS& MakeParms,
			const COLOR_ARRAY& cmap,		//!< Initial colormap
			const char* Source = 0
			);

		ERRVALUE Open (
			const RVC::OBJITEM& ObjItem
			);

		void SetColorPolicy (
			COLORPOLICY policy
			) { m_ColorPolicy = policy; }

		//! Set the pixel value to fallback to if using COLORPOLICY_Fallback
		void SetFallbackPixel (
			PIXEL pixel
			) { m_FallbackPixel = pixel; }
			
	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class CONTEXT_RASTER_COLORMAPPED;

		//! Copy constructor private and unimplemented
		DEVICE_RASTER_COLORMAPPED(const DEVICE_RASTER_COLORMAPPED&);

		COLOR_ARRAY m_ColorMap;
		bool m_bColorMapDirty;
		COLOR m_LastColor;	//!< For speeding up GetColorPixel()
		PIXEL m_LastPixel;	//!< For speeding up GetColorPixel()
		PIXEL m_FallbackPixel;
		COLORPOLICY m_ColorPolicy;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Create a GC for drawing.  
		//!
		//! Derived classes may override this and make it return anything
		//! derived from MGD_GC.  MGD_GC's constructor is private, so this
		//! is the only way for calling code to create a GC.
		virtual CONTEXT* v_CreateContext (
			);

		virtual const COLOR_ARRAY* v_GetColorMap (
			) const;

		virtual PIXEL v_GetColorPixel (
			const COLOR& color
			);

		virtual COLORSPACE v_GetColorSpace (
			) const {
			//! We fake it and look up the color later
			return (COLORSPACE_R8_G8_B8);
			}

		virtual void v_GetPixelColor (
			PIXEL pixel,
			COLOR& color
			) const;

		virtual ERRVALUE v_Flush();
	};

class CONTEXT_RASTER_COLORMAPPED : public CONTEXT_RASTER_GRAY8 {
	public:
		// Note, default Constructor is protected

		virtual ~CONTEXT_RASTER_COLORMAPPED();


	protected:
		//! Default Constructor 
		CONTEXT_RASTER_COLORMAPPED (
			DEVICE_RASTER_COLORMAPPED* dev
			);

		DEVICE_RASTER_COLORMAPPED* m_DevRastC;

		virtual void v_UpdateColor();

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Copy constructor private and unimplemented
		CONTEXT_RASTER_COLORMAPPED(const CONTEXT_RASTER_COLORMAPPED&);

		friend class MGD::DEVICE_RASTER_COLORMAPPED;	// Only DEVICE_RASTER_COLORMAPPED::CreateContext() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT
	
	};


} // End of namespace MGD


#endif	// INC_MI32_MG2RAST_H

