/**
 * \file rvc/georefer.h 
 * \brief interface for the RVC::GEOREFERENCE class.
 *
 * \if NODOC
 * $Id: georefer.h_v 1.36 2011/12/23 15:37:14 mju Exp $
 *
 * $Log: georefer.h_v $
 * Revision 1.36  2011/12/23 15:37:14  mju
 * Use trans2daffine hdr.
 *
 * Revision 1.35  2011/12/09 22:50:38  mju
 * Remove redundant hdr.
 *
 * Revision 1.34  2007/09/06 17:18:48  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.33  2007/06/15 16:37:01  mju
 * Add setPointList from array/count.
 *
 * Revision 1.32  2005/12/09 17:29:30  scowan
 * Updated to use credentials.
 *
 * Revision 1.30  2005/04/05 22:11:36  scowan
 * Chagne set CRS method to validatre CRS and return error if needed.
 *
 * Revision 1.26  2005/03/03 23:57:34  scowan
 * Allow method to get affine matrix.
 *
 * Revision 1.23  2004/06/21 17:19:21  scowan
 * Added coord ref sys methods.
 *
 * Revision 1.20  2004/05/11 23:07:40  scowan
 * Added boundary methods.
 *
 * Revision 1.19  2004/01/28 23:02:36  scowan
 * Change 3d sheet to mani fold.
 *
 * Revision 1.18  2004/01/20 23:22:02  scowan
 * Updated 3dsheet.
 *
 * Revision 1.17  2003/12/19 21:07:45  scowan
 * Finialized (for now) the new georefernce interface.
 *
 * Revision 1.9  2003/07/23 20:11:48  mju
 * Add GEOREFTYPE_AffineMatrix so meaning clearer.
 *
 * Revision 1.7  2001/09/05 21:05:33  scowan
 * Added conversion methods on the source coordinates.
 *
 * Revision 1.1  2000/09/29 16:25:29  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_GEOREFER_H
#define INC_RVC_GEOREFER_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_TRANS2DAFFINE_H
	#include <mi32/trans2daffine.h>
#endif

#ifndef INC_MI32_TRANSMOD_H
	#include <mi32/transmod.h>
#endif

#ifndef INC_MI32_SPATREF_H
	#include <mi32/spatref.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class TRANS2D_MAPGEN;
namespace RVC {
	class GEOREFHANDLE;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_Georeference RVC objects.  
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Georeference.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_TIN
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_Hyperspectral
//!
//!	Read() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods
class GEOREFERENCE : public SIMPLE {  
	public:

		//! Type of georeference
		enum GEOREFTYPE {
			GEOREFTYPE_Implied =			5,			//!< Georeference information is implied in the data
			GEOREFTYPE_AffineMatrix =	6,			//!< Affine transformation matrix
			GEOREFTYPE_Matrix =	GEOREFTYPE_AffineMatrix,
			GEOREFTYPE_CtrlPoint =		7,			//!< List of CTRLPOINT3's
			GEOREFTYPE_Manifold = 		8			//!< Set of CTRLPOINT3's to create a Manifold on
			};

		struct CTRLPTEDGE {
			INT32 m_StartCtrlPtID;
			INT32 m_EndCtrlPtID;
			};
	
		//! Default Constructor
		GEOREFERENCE ();

		//! Copy Constructor
		GEOREFERENCE (
			const RVC::GEOREFERENCE& rhs					
			);										

		//! Destructor
		virtual ~GEOREFERENCE ();										

		//! Assignment
		RVC::GEOREFERENCE& operator= (
			const RVC::GEOREFERENCE& rhs
			);

		//! Add a georeference boundary control point array to the list of boundary control point arrays
		//! Only works for extented georeference objects
		//! @return Number of the added boundary control point array
		INT32 AddBoundary (
			const SIMPLE_ARRAY<INT32>& Boundary
			);
			
		//! Add a georeference control point to the list of control points
		//! Works for either basic or extented georeference objects
		//! @return Number of the added control point
		INT32 AddCtrlPoint (
			const CTRLPOINT3& CtrlPoint
			);
			
		//! Add a georeference control point edge entry to the list of control points edge entries
		//! Only works for extented georeference objects
		//! @return Number of the added control point edge entry
		INT32 AddCtrlPointEdge (
			const CTRLPTEDGE& CtrlPtEdge
			);

		//! Clear out all georeference control point array entries
		//! Only works for extented georeference objects
		ERRVALUE ClearBoundaries (
			);
			
		//! Clear out all georeference control point entries
		//! Works for either basic or extented georeference objects.
		ERRVALUE ClearCtrlPoints (
			);
			
		//! Clear out all georeference control point edge entries
		//! Only works for extented georeference objects
		ERRVALUE ClearCtrlPointEdges (
			);
			
		//! Convert control point list 'source' coordinates using an affine transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertForward (
			const TRANS2D_AFFINE& Affine
			);
		
		//! Convert control point list 'source' coordinates using an affine transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertInverse (
			const TRANS2D_AFFINE& Affine
			);
		
		//! Convert control point list 'source' coordinates using an TRANS2D_MAPGEN transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertForward (
			const TRANS2D_MAPGEN& tmg
			);
		
		//! Convert control point list 'source' coordinates using an TRANS2D_MAPGEN transformation
		//! Only works for internally stored ctrl point lists, does not affect disabled ctrl points
		ERRVALUE ConvertInverse (
			const TRANS2D_MAPGEN& tmg
			);

		//! Convert a basic georeference object to an extented georeference object		
		//! Only works for basic georeference objects
		ERRVALUE ConvertToExtended (
			);
			
		//! Delete a control point at the location specified
		//! Maintains the order of the control point list
		//! Works for either basic or extented georeference objects.
		ERRVALUE DeleteCtrlPoint (
			INT32 CtrlPointNum
			);
			
		//! Delete a control point edge entry at the location specified
		//! Maintains the order of the control point edge list
		//! Only works for extented georeference objects
		ERRVALUE DeleteCtrlPointEdge (
			INT32 CtrlPointEdgeNum
			);
			
		//! Get georeference affine matrix.  Only value if GetGeorefType() == GEOREFTYPE_AffineMatrix
		//! @return Affine matrix
		const TRANS2D_AFFINE& GetAffineMatrix (
			) const;

		//! Obtain the calibration / translation model from the rvc georeference object
		//! @return calibration / translation model class instance
		TRANSMODEL GetCalibModel (
			) const;

		//! Get georeference Coordinate reference system parameters
		//! @return Coordinate reference system parameters to use
		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const;

		//! Obtain the georeference type
		//! @return Georeference Type
		GEOREFTYPE GetGeorefType (
			) const;

		//! Generate a TRANS2D_MAPGEN from the georeference instance
		ERRVALUE GetMapTrans (
			TRANS2D_MAPGEN& tmg,							//!< Generic transformation generated and RETURNED
			bool DoOutput = false,						//!< Place transformation on output side if true
			TRANSMODEL model = 0							//!< Translation model, zero sets up the default model
			) const;

		//! Get the number of boundary control point array items
		//! Only works for extented georeference objects
		//! @return Number of boundary control point array entries
		INT32 GetNumBoundaries (
			);
			
		//! Get the number of control points
		//! Works for either basic or extented georeference objects
		//! @return Number of control points
		INT32 GetNumCtrlPoints (
			);
			
		//! Get the number of control points edge entries
		//! Only works for extented georeference objects
		//! @return Number of control point edge entries
		INT32 GetNumCtrlPointEdges (
			);
			
		//! Generate a control point list from the georeference
		//! This method will generate control points if the georeference is not a control point georeference
		ERRVALUE GetPointList (
			SIMPLE_ARRAY<CTRLPOINT3>& CtrlList		//!< Control point list allocated and returned
			) const;

		//! Insert a control point at the location specified
		//! Maintains the order of the control point list
		//! Works for either basic or extented georeference objects.
		ERRVALUE InsertCtrlPoint (
			INT32 CtrlPointNum,
			const CTRLPOINT3& CtrlPoint
			);
			
		//! Insert a control point edge entry at the location specified
		//! Maintains the order of the control point edge list
		//! Works for extented georeference objects only.
		ERRVALUE InsertCtrlPointEdge (
			INT32 CtrlPointEdgeNum,
			const CTRLPTEDGE& CtrlPtEdge
			);
			
		//! Determine if the georeference object is basic or extented
		//! @return 'True' if the georeference object is basic, 'false' if extented
		bool IsBasicFormat (
			);
			
		//! Determine if the referenced control point is enabled
		//! Works for either basic or extented georeference objects.  Standard georeference objects always return 'true'
		//! @return 'True' if the control point is enabled, 'false' if not
		bool IsCtrlPointEnabled (
			INT32 CtrlPointIndex
			);

		//! Determine if the two georeference objects are equivalent
		//! @return 'True' if the georeference information is equivalent, 'false' if not
		bool IsEquivalentTo (
			const RVC::GEOREFERENCE& GeorefObj
			) const;
			
		//! Read a specific georeference boundary control point array entry
		//! Only works for extented georeference objects
		ERRVALUE ReadBoundary (
			INT32 BoundaryIndex,
			SIMPLE_ARRAY<INT32>& Boundary
			);
			
		//! Read a specific georeference control point
		//! Works for either basic or extented georeference objects.
		ERRVALUE ReadCtrlPoint (
			INT32 CtrlPointIndex,
			CTRLPOINT3& CtrlPoint
			);
			
		//! Read a specific georeference control point edge entry
		//! Only works for extented georeference objects
		ERRVALUE ReadCtrlPointEdge (
			INT32 CtrlPointEdgeIndex,
			CTRLPTEDGE& CtrlPtEdge
			);
			
		//! Read a specific georeference control point ID entry
		//! Enhanced georeference objects can store an ID value associated with the control point
		//! Only works for extented georeference objects
		INT32 ReadCtrlPointID (
			INT32 CtrlPointIndex
			);
			
		//! Read a specific georeference control point name entry
		//! Enhanced georeference objects can store a name value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE ReadCtrlPointName (
			INT32 CtrlPointIndex,
			MISTRING& CtrlPointName
			);
			
		//! Set the number of control point edge entries 
		//! Only works for extented georeference objects
		ERRVALUE ResizeCtrlPointEdges (
			INT32 NumCtrlPointEdges
			);
			
		//! Set the number of control points 
		//! Works for either basic or extented georeference objects.
		ERRVALUE ResizeCtrlPoints (
			INT32 NumCtrlPoints
			);
			
		//! Set georeference to be a Manifold georeference
		void SetManifold (
			);

		//! Set georeference to be an affine transformation
		void SetAffine (
			const TRANS2D_AFFINE& taf					//!< Affine transformation to set in georeference
			);

		//! Set calibration model
		void SetCalibModel (						
			const TRANSMODEL& model						//!< Calibration / translation model to use
			);

		//! Set georeference Coordinate reference system parameters
		ERRVALUE SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys					//!< Coordinate reference system parameters to use
			);

		//! Set the specified control point to be enabled or disabled
		//! Only works for extented georeference objects
		ERRVALUE SetCtrlPointEnabled (
			INT32 CtrlPointIndex,
			bool IsEnabled
			);
			
		//! Set georeference to be an implied transformation
		void SetImplied (
			);

		//! Set georeference to be an control point transformation
		ERRVALUE SetPointList (
			const SIMPLE_ARRAY<CTRLPOINT3>& CtrlList	//!< Control point list to use
			);
			
		//! Set georeference to be an control point transformation
		ERRVALUE SetPointList (
			const CTRLPOINT3 *CtrlPtList,					//!< Control point list to use
			int NumCtrlPts										//!< Number of control points
			);
			
		//! Write a specific georeference boundary control point array entry
		//! Only works for extented georeference objects
		ERRVALUE WriteBoundary (
			INT32 BoundaryIndex,
			const SIMPLE_ARRAY<INT32>& Boundary
			);
			
		//! Write a specific georeference control point.  Use AddCtrlPoint() to add control points to the list
		//! Works for either basic or extented georeference objects.
		ERRVALUE WriteCtrlPoint (
			INT32 CtrlPointIndex,
			const CTRLPOINT3& CtrlPoint
			);
			
		//! Write a specific georeference control point edge entry
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointEdge (
			INT32 CtrlPointEdgeIndex,
			const CTRLPTEDGE& CtrlPtEdge
			);
			
		//! Write a specific georeference control point name entry
		//! Enhanced georeference objects can store an ID value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointID (
			INT32 CtrlPointIndex,
			INT32 CtrlPointID
			);
			
		//! Write a specific georeference control point name entry
		//! Enhanced georeference objects can store a name value associated with the control point
		//! Only works for extented georeference objects
		ERRVALUE WriteCtrlPointName (
			INT32 CtrlPointIndex,
			const MISTRING& CtrlPointName
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;					
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		
		ERRVALUE ReadStandardGeorefData ();
		ERRVALUE UpdateHeader ();

	 	GEOREFTYPE m_GeorefType;					//!< Type of georeference used
		TRANSMODEL m_CalibModel;					//!< Calibration model using TRANS2DMODEL values defined in <mi32/transmod.h>
		SPATREF::COORDREFSYS m_CoordRefSys;		//!< Coordinate reference system of the georeference
		SIMPLE_ARRAY<CTRLPOINT3> m_CtrlList;	//!< Control point list if georeference has one
		TRANS2D_AFFINE m_Affine;					//!< Affine transformation if georeference has one
		bool m_ChangedType;							//!< Type of projection (Affine, Implied, Ctrl Point) and or projection name has changed
		GEOREFHANDLE* m_GeorefHandle;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
}		// End of RVC namespace

#endif	// INC_RVC_GEOREFER_H
