/**
 * \file rvc/list.h 
 * \brief interface for the RVC::LIST class.
 *
 * \if NODOC
 * $Id: list.h_v 1.18 2011/12/21 22:14:58 mju Exp $
 *
 * $Log: list.h_v $
 * Revision 1.18  2011/12/21 22:14:58  mju
 * Remove redundant hdr.
 *
 * Revision 1.17  2007/04/03 14:49:21  scowan
 * Moved genlist info into rvc namespace.
 *
 * Revision 1.14  2006/02/01 22:52:11  scowan
 * Added methods to read and write double array's.
 *
 * Revision 1.1  2001/05/24 16:11:59  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_LIST_H
#define INC_RVC_LIST_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct GENLISTINFO;
class LISTHANDLE;
#endif


//! Manages RVC::OBJTYPE_GenericRef objects.  
//!	
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
class LIST : public RVC::OBJECT {
	public:

		//! Default constructor
		LIST (
			);

		//! Copy constructor
		LIST (
			const RVC::LIST& obj
			);

		//! Destructor
		virtual ~LIST (
			);

		//! Assignment.  Will close if necessary and reopen if 'rhs' is open
		RVC::LIST& operator= (
			const RVC::LIST& rhs
			);

		//! Obtain the size of a list entry
		//! @return List entry size	
		INT32 GetItemSize (
			INT32 ItemNum
			) const;
			
		//! Obtain the size of the largest list entry
		//! @return Largest list entry size	
		INT32 GetMaxItemSize (
			) const;
			
		//! Obtain the number of list entries
		//! @return Number of list entries
		INT32 GetNumItems (
			) const;
		
		//! Inserts 1 or more entries into a generic list with each new entry initially the size of zero
		ERRVALUE InsertItems (
			INT32 StartPosn,
			INT32 NumItems = 1
			);

		//! Create a new rvc list object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			const RVC::OBJECT& parent,					//!< Parent object to create list object under
			const RVC::DESCRIPTOR& Descriptor,		//!< Name of new list object
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new rvc list object under a specified parent rvc object with name, description, and source
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
			const CREDENTIALS* Credentials 
#else
			const char* Source = 0,						//!< Source of new list object, default is application name
			const CREDENTIALS* Credentials = 0		//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read a list entry
		//! @return Number of bytes of the entry		
		INT32 Read (
			INT32 ItemNum,
			void *ptr
			) const;

		//! Read a list entry
		//! @return Number of bytes of the entry		
		INT32 Read (
			INT32 ItemNum,
			MIDOUBLEARRAY& ptr
			) const;

		//! Read a list entry
		//! @return Number of bytes of the entry		
		INT32 Read (
			INT32 ItemNum,
			void **buffer,
			INT32 BufferSize
			) const;

		//! Read a list entry into a SIMPLE_ARRAY
		template <class _CT>
		ERRVALUE Read (
			INT32 ItemNum,
			SIMPLE_ARRAY<_CT>& data
			) const {
			INT32 MaxItems = data.GetMaxItems();
			_CT* buffer = data.Detach();
			INT32 NumBytes = Read(ItemNum, reinterpret_cast<void**>(&buffer), MaxItems * sizeof(_CT));
			if (NumBytes < 0) {
				data.Attach(buffer, MaxItems);
				data.Clear();
				}
			else {
				if (static_cast<UINT32>(NumBytes) > MaxItems * sizeof(_CT)) MaxItems = NumBytes / sizeof(_CT);
				data.Attach(buffer, MaxItems);
				data.Resize(NumBytes / sizeof(_CT), true, false);	//! Set to correct number of items
				NumBytes = 0;
				}
			return (NumBytes);
			}
			
		//! Read a list entry into a DOUBLE_ARRAY
		template <class _CT>
		ERRVALUE Read (
			INT32 ItemNum,
			DOUBLE_ARRAY<_CT>& data
			) const {
			INT32 MaxItems = data.GetMaxItems();
			MIDOUBLEARRAY items;
			data.Detach(items);
			INT32 NumBytes = Read(ItemNum, items);
			if (NumBytes < 0) {
				data.Attach(items, MaxItems);
				data.Clear();
				}
			else {
				if (static_cast<UINT32>(NumBytes) > MaxItems * sizeof(_CT)) MaxItems = NumBytes / sizeof(_CT);
				data.Attach(items, MaxItems);
				data.Resize(NumBytes / sizeof(_CT), true, false);	//! Set to correct number of items
				NumBytes = 0;
				}
			return (NumBytes);
			}
			
		//! Partially read a list entry
		//! @return Number of bytes read of the entry		
		INT32 Read (
			INT32 ItemNum,
			INT32 StartPosn,
			INT32 Length,
			void *ptr
			) const;

		//! Read a list entry into a SIMPLE_ARRAY
		template <class _CT>
		ERRVALUE Read (
			INT32 ItemNum,
			INT32 StartPosn,
			INT32 Length,			//!< Length in _CT units
			SIMPLE_ARRAY<_CT>& data
			) const {
			INT32 NumBytes = Read(ItemNum, StartPosn * sizeof(_CT), Length * sizeof(_CT), static_cast<void*>(data));
			if (NumBytes >= 0) {
				data.Resize(NumBytes / sizeof(_CT), true, false);	//! Set to correct number of items
				NumBytes = 0;
				}
			return (NumBytes);
			}
			
		//! Resize the list to a new number of items
		ERRVALUE Resize (
			INT32 NewNumItems
			);

		//! Resize a list item to a new size		
		ERRVALUE ResizeItem (
			INT32 ItemNum,
			INT32 NewItemSize
			);
			
		//! Swap two list items
		ERRVALUE SwapItems (
			INT32 Item1,
			INT32 Item2
			);
			
		//! Write data to a list item
		//! @return Number of bytes written
		INT32 Write (
			INT32 ItemNum,
			INT32 Length,
			const void* ptr
			);
			
		//! Write partial data to a list item
		//! @return Number of bytes written
		INT32 Write (
			INT32 ItemNum,
			INT32 TotalSize,
			INT32 StartPosn,
			INT32 Length,
			const void* ptr
			);
			
		//! Write SIMPLE_ARRAY<> to a list item
		template <class _CT>
		ERRVALUE Write (
			INT32 ItemNum,
			const SIMPLE_ARRAY<_CT>& data
			) {
			ERRVALUE err = Write(ItemNum, data.GetNumItems() * sizeof(_CT), data);
			if (err > 0) err = 0;
			return (err);
			}
			
		//! Write SIMPLE_ARRAY<> to a list item
		template <class _CT>
		ERRVALUE Write (
			INT32 ItemNum,
			const DOUBLE_ARRAY<_CT>& data
			) {
			ERRVALUE err = Write(ItemNum, data.GetNumItems() * sizeof(_CT), data);
			if (err > 0) err = 0;
			return (err);
			}
			
	protected:		
		
		//! Used for objects that inherit from this object
		ERRVALUE Make (const OBJECT& parent, const DESCRIPTOR& Descriptor, GENLISTINFO& newinfo, int HeaderSize, int HeaderType, const CREDENTIALS* Credentials);

		//! Used for objects that inherit from this object
		ERRVALUE OpenObject (OPENMODE OpenFlags, GENLISTINFO& newinfo, int HeaderSize, int HeaderType, MDLGPARENT parent, const CREDENTIALS* Credentials);
		
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();				//! Close the open object
		virtual OBJTYPE v_GetDftObjectType () const;		//!< Get the default object type
		virtual bool v_IsObjectOpen () const;					//!< Is this object opened?
		virtual ERRVALUE v_OpenObject (OPENMODE OpenFlags, MDLGPARENT parent, const CREDENTIALS* Credentials);				//! Open the object

		LISTHANDLE* m_ListHandle;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_LIST_H
