/**
 * \file rvc/rasterthemeparm.h
 * \brief RVC::RASTERTHEMEPARM class definition
 *
 * \if NODOC
 * $Id: rasterthemeparm.h_v 1.5 2012/04/24 17:25:34 mju Exp $
 *
 * $Log: rasterthemeparm.h_v $
 * Revision 1.5  2012/04/24 17:25:34  mju
 * Inline get and most set methods.
 * Don't return struct/class instance by value.
 *
 * Revision 1.4  2008/09/02 17:47:09  mju
 * Fix doxygen warnings.
 *
 * Revision 1.3  2007/09/10 14:53:41  scowan
 * Added line at end to remove silly g++ warnings.
 *
 * Revision 1.2  2007/09/06 17:18:56  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.1  2007/07/05 15:39:36  honerkamp
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_RASTERTHEMEPARM_H
#define INC_RVC_RASTERTHEMEPARM_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif


namespace RVC {
class OBJMAKEPARMS;     // Forward Declaration

class RASTERTHEMEPARM : public RVC::SIMPLE {
	public:

		enum DISTRIB {
			DISTRIB_UserDefined = 0,
			DISTRIB_EqualCount,
			DISTRIB_EqualInterval,
			};

		enum ROUNDMODE {
			ROUNDMODE_None = 0,
			ROUNDMODE_Closest,
			ROUNDMODE_Down,
			ROUNDMODE_Up
			};

		enum COLORPATH {
			COLORPATH_Auto = 0,
			COLORPATH_RGB,
			COLORPATH_HISCW,
			COLORPATH_HISCCW,
			COLORPATH_HBSCW,
			COLORPATH_HBSCCW,
			COLORPATH_MaxAuto = COLORPATH_HBSCCW
			};

		struct SPREAD {
			INT16 MinClass;						//!< Starting class for spread
			INT16 MaxClass;						//!< Ending class for spread
			UINT8 AllClasses;						//!< Set if applies to all classes
			UINT8 DoColor;							//!< Perform color spread
			COLORPATH ColorPath;					//!< Path for color spread
			COLOR StartColor;						//!< Starting spread color
			COLOR EndColor;						//!< Ending spread color
			};

		//! Default constructor
		RASTERTHEMEPARM ();

		//! Copy constructor
		RASTERTHEMEPARM (
			const RVC::RASTERTHEMEPARM& rhs
			);

		//! Destructor
		virtual ~RASTERTHEMEPARM ();

		//! Assignment
		RVC::RASTERTHEMEPARM& operator= (
			const RVC::RASTERTHEMEPARM& rhs
			);

		//! Get Number of classes in the distribution. Note it will be one less than the number of class boundaries
		INT16 GetNumClasses () const
			{ return (m_NumClasses); }

		//! For floating point rasters set true if the cell on the boundary is included in the upper range
		bool IsUpperInclusive () const
			{ return (m_IsUpperInclusive); }

		//! Distribution mode, DISTRIB_...
		DISTRIB GetDistribMode () const
			{ return (m_DistribMode); }

		//! Rounding mode, ROUNDMODE_...
		ROUNDMODE GetRoundMode () const
			{ return (m_RoundMode); }

		//! Exclude first class from distribution
		bool IsFirstClassExcluded () const
			{ return (m_ExcludeFirstClass); }

		//! Exclude last class from distribution
		bool IsLastClassExcluded () const
			{ return (m_ExcludeLastClass); }

		//! Limit class value ranges to existing data range
		bool IsLimitedToDataRange () const
			{ return (m_LimitToDataRange); }

		//! Interval to round to
		double GetRoundTo () const
			{ return (m_RoundTo); }

		//! Boundaries of the ranges for the classes
		const SIMPLE_ARRAY<double>& GetClassBoundaries () const
			{ return (m_ClassBoundaries); }

		const SPREAD& GetSpread () const
			{ return (m_Spread); }

		//! Set the number of classes
		void SetNumClasses (INT16 numclasses)
			{ m_NumClasses = numclasses; }

		//! Set Whether the range is upper inclusive
		void SetUpperIsInclusive (bool isupperinclusive)
			{ m_IsUpperInclusive = isupperinclusive; }

		//! Set the distribution mode
		void SetDistribMode (DISTRIB distribmode)
			{ m_DistribMode = distribmode; }

		//! Set the Rounding mode, ROUNDMODE_...
		void SetRoundMode (ROUNDMODE roundmode)
			{ m_RoundMode= roundmode; }

		//! Set whether to Exclude first class from distribution
		void SetExcludeFirstClass (bool excludefirstclass)
			{ m_ExcludeFirstClass = excludefirstclass; }

		//! Set whether to Exclude last class from distribution
		void SetExcludeLastClass (bool excludelastclass)
			{ m_ExcludeLastClass = excludelastclass; }

		//! Set Limit class value ranges to existing data range
		void SetLimitToDataRange (bool limittorange)
			{ m_LimitToDataRange = limittorange; }

		//! Set Interval to round to
		void SetRoundTo (double roundto)
			{ m_RoundTo = roundto; }

		//! Set Boundaries of the ranges for the classes
		void SetClassBoundaries (const SIMPLE_ARRAY<double> &boundaries);

		//! Set the color spread
		void SetSpread (const SPREAD &spread);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_IsUpperInclusive;						// For floating point rasters set true if you want the cell on the boundary to be included in the upper range
		INT16 m_NumClasses;								// Number of classes
		DISTRIB m_DistribMode;							// Distribution mode, DISTRIB_...
		ROUNDMODE m_RoundMode;							// Rounding mode, ROUNDMODE_...
		bool m_ExcludeFirstClass;						// Exclude first class from distribution
		bool m_ExcludeLastClass;						// Exclude last class from distribution
		bool m_LimitToDataRange;						// Limit class value ranges to existing data range
		double m_RoundTo;									// Interval to round to
		double m_lastmaxval;								// Last maximum range value
		SIMPLE_ARRAY<double> m_ClassBoundaries;  // Boundaries of the ranges for the classes
		SPREAD m_Spread;

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		// Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
}		// End of RVC namespace

#endif 	// INC_RVC_RASTTHEMEPRM_H

