/**
 * \file rvc/serialze.h 
 * \brief interface for the RVC::SERIALIZE class.
 *
 * \if NODOC
 * $Id: serialze.h_v 1.16 2006/09/08 19:28:23 scowan Exp $
 *
 * $Log: serialze.h_v $
 * Revision 1.16  2006/09/08 19:28:23  scowan
 * Change enum from serialize to its own enum.
 *
 * Revision 1.15  2006/09/08 14:35:35  mju
 * Add filter_usageforobjtype.
 *
 * Revision 1.14  2006/06/27 15:04:51  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.13  2005/12/19 18:36:20  dwilliss
 * adjusted mandatory credentials to account for prior optional parameters
 *
 * Revision 1.12  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.11  2005/12/09 17:29:53  scowan
 * Updated to use credentials.
 *
 * Revision 1.10  2005/07/01 21:06:35  scowan
 * Do not make dtor pure.
 *
 * Revision 1.9  2005/06/30 21:02:21  scowan
 * Added makeparms.
 *
 * Revision 1.8  2005/01/19 23:02:25  scowan
 * Added support for descriptor.
 *
 * Revision 1.7  2003/10/07 17:30:42  scowan
 * Object virtual method changes.
 *
 * Revision 1.6  2003/10/06 15:43:15  scowan
 * Made virtual methods const to match base classes.
 *
 * Revision 1.5  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.3  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2001/06/15 14:21:28  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/29 17:06:44  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::SERIALIZE rvc/serialze.h
//!	\brief Interface for RVC objects that support a serialize text object paradigm.
//!	This includes methods to get usage and object type information.
//!
//!	RVC Object Types that this class supports: Derived class
//!
//!	Default RVC Object Type for this class: Derived class
//!
//!	Valid parents for this class: Derived class

#ifndef INC_RVC_SERIALZE_H
#define INC_RVC_SERIALZE_H

#ifndef INC_RVC_TEXT_H
	#include <rvc/textobj.h>
#endif

#ifndef INC_RVC_DISPPARMUSAGE_H
	#include <rvc/dispparmusage.h>
#endif

namespace RVC {

class SERIALIZE : public RVC::TEXTOBJ {
	public:
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		class FILTER_USAGEFOROBJTYPE;
		#endif
		
		//! Default constructor
		SERIALIZE (
			);

		//! Copy Constructor
		SERIALIZE (
			const RVC::SERIALIZE& obj
			);

		//! Destructor
		virtual ~SERIALIZE (
			);

		//! Assignment
		RVC::SERIALIZE& operator= (
			const RVC::SERIALIZE& rhs
			);

		//! Obtain the usage of the serialize object
		//! @return DISPPARMUSAGE_...
		DISPPARMUSAGE GetUsage ( 
			) const;

		//! Obtain version number of seralize object
		//! @return Version number
		UINT16 GetVersion (
			) const;

		//! Obtain type of object this serialize object belongs to
		//! @return RVCOBJECT::OBJTYPE
		RVC::OBJTYPE GetForObjectType (
			) const;

		//! Get correct OBJTYPESET for all text serialize OBJTYPE's
		//! @return OBJTYPESET for all text serialize OBJTYPE's
		static RVC::OBJTYPESET GetObjTypeSet (
			);
			
		//! Determine if the object type is a script text object
		static bool IsSerialize (
			RVC::OBJTYPE ObjType
			);
			
		//! Create serialize object
		ERRVALUE Make (
	    	const RVC::OBJECT& parent,					//!< Parent object to create metadata under
			const RVC::DESCRIPTOR& Descriptor,
			const MAKEPARMS& MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS		//!< User credentials passed in, NULL for default behavior
			);

		//! Create serialize object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,						//!< Source of raster creation, default is GetAppName()
			const CREDENTIALS* Credentials 	//!< User credentials passed in, NULL for default behavior
#else
			const char* Source = 0,						//!< Source of raster creation, default is GetAppName()
			const CREDENTIALS* Credentials = 0		//!< User credentials passed in, NULL for default behavior
#endif
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);
		virtual bool v_ValidateObjectType (const OBJTYPE otype) const;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! Container for parameters to make object.
class SERIALIZE::MAKEPARMS : public TEXTOBJ::MAKEPARMS {
	public:
		MAKEPARMS (
			RVC::OBJTYPE ObjType,
			UINT16 Version = 1,				 			//!< Version number to assign to new object
			OBJTYPE ForObjType = OBJTYPE_All,		//!< RVC object that this serialize object is referenced by
			DISPPARMUSAGE Usage = DISPPARMUSAGE_Display			//!< Usage of this object
			);
			
		//! Obtain the usage of the serialize object
		//! @return DISPPARMUSAGE_...
		DISPPARMUSAGE GetUsage (
			) const;

		//! Obtain version number of seralize object
		//! @return Version number
		UINT16 GetVersion (
			) const;

		//! Obtain type of object this serialize object belongs to
		//! @return RVC::OBJTYPE
		RVC::OBJTYPE GetForObjectType (
			) const;

		//! Set the usage of the serialize object
		void SetUsage (
			DISPPARMUSAGE usage
			);

		//! Set the version number of seralize object
		void SetVersion (
			UINT16 version
			);

		//! Set the type of object this serialize object belongs to
		void SetForObjectType (
			RVC::OBJTYPE objtype
			);

	};


class SERIALIZE::FILTER_USAGEFOROBJTYPE : public RVC::OBJECT::FILTER {
	public:
		//! Constructor with usage and optional object type.
		FILTER_USAGEFOROBJTYPE (
			DISPPARMUSAGE usage,								//!< Allowed usage
			OBJTYPE forobjtype = OBJTYPE_All		//!< Allowed values of associated object type, OBJTYPE_All for any
			): m_usage(usage), m_forobjtype(forobjtype) { }
		//! Set allowed object type.
		void SetForObjType (
			OBJTYPE forobjtype
			) { m_forobjtype = forobjtype; }
		//! Set allowed usage.
		void SetUsage (
			DISPPARMUSAGE usage
			) { m_usage = usage; }
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DISPPARMUSAGE m_usage;
		OBJTYPE m_forobjtype;
		virtual bool Filter (const RVC::OBJECT& obj);
		#endif
	};

}		//! End of RVC namespace

#endif 	//!< INC_RVC_SERIALZE_H
