/**
 * \file spatmod/datatype.h
 * \brief Spatial Modelling (SPATMOD) DATATYPE definitions
 *
 * \if NODOC
 * $Id: datatype.h_v 1.8 2012/06/01 21:37:07 mju Exp $
 *
 * $Log: datatype.h_v $
 * Revision 1.8  2012/06/01 21:37:07  mju
 * Add dataTypeOf functions for use in templates.
 *
 * Revision 1.7  2010/09/27 14:06:19  mju
 * Add getDataTypeCombined.
 *
 * Revision 1.6  2008/08/28 13:53:37  mju
 * Fix doxygen file ref.
 *
 * Revision 1.5  2006/08/04 13:58:42  mju
 * Add getNextDataType.
 *
 * Revision 1.4  2006/01/23 20:50:36  mju
 * Add statsvalidity enum.
 *
 * Revision 1.3  2005/12/15 20:54:40  mju
 * Declare georeference class.
 *
 * Revision 1.2  2005/08/22 17:12:26  mju
 * Add rangetype enum.
 *
 * Revision 1.1  2005/08/16 15:08:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_DATATYPE_H
#define	INC_SPATMOD_DATATYPE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace SPATMOD {

//! Enumeration of data types which can be manipulated as a single value in
//! memory by the CPU and can be declared as built-in type in source code.
enum DATATYPE {
	DATATYPE_Invalid =				0,
	DATATYPE_Unsigned =				0x1000,
	DATATYPE_Signed =					0x2000,
	DATATYPE_Float =					0x4000,
	DATATYPE_UINT8 =					(DATATYPE_Unsigned|1),
	DATATYPE_UINT16 =					(DATATYPE_Unsigned|2),
	DATATYPE_UINT32 =					(DATATYPE_Unsigned|4),
	DATATYPE_UINT64 =					(DATATYPE_Unsigned|8),
	DATATYPE_SINT8 =					(DATATYPE_Signed|1),
	DATATYPE_SINT16 =					(DATATYPE_Signed|2),
	DATATYPE_SINT32 =					(DATATYPE_Signed|4),
	DATATYPE_SINT64 =					(DATATYPE_Signed|8),
	DATATYPE_FLOAT32 =				(DATATYPE_Float|4),
	DATATYPE_FLOAT64 =				(DATATYPE_Float|8),
	DATATYPE_MASK_Size =				0x00FF,
	DATATYPE_MASK_Type =				0x7000
	};

//! Enumeration of data range types.
//! Used to specify meaning of range values.
enum RANGETYPE {
	RANGETYPE_Default,						//!< Range is default for datatype/precision
	RANGETYPE_Exact,							//!< All data values inside range, exact range of data values known
	RANGETYPE_Maximal,						//!< All data values inside range, may be larger than exact extent of data values
	RANGETYPE_Typical							//!< Typical data value is inside range, may be some valid data values outside range
	};

//! Enumaration for statistics validity.
enum STATSVALIDITY {
	STATSVALIDITY_None,						//!< No statistics available
	STATSVALIDITY_Fastest,					//!< Compute(d) with fastest method, may not equally sample entire data domain
	STATSVALIDITY_Approximate,				//!< Compute(d) as approximation, may be sample of entire valid data domain
	STATSVALIDITY_Exact						//!< Compute(d) as exact, processes all valid data samples
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
class GEOREFERENCE;
#endif

inline DATATYPE DataTypeOf (UINT8)
	{ return (DATATYPE_UINT8); }

inline DATATYPE DataTypeOf (UINT16)
	{ return (DATATYPE_UINT16); }

inline DATATYPE DataTypeOf (UINT32)
	{ return (DATATYPE_UINT32); }

inline DATATYPE DataTypeOf (UINT64)
	{ return (DATATYPE_UINT64); }

inline DATATYPE DataTypeOf (INT8)
	{ return (DATATYPE_SINT8); }

inline DATATYPE DataTypeOf (INT16)
	{ return (DATATYPE_SINT16); }

inline DATATYPE DataTypeOf (INT32)
	{ return (DATATYPE_SINT32); }

inline DATATYPE DataTypeOf (INT64)
	{ return (DATATYPE_SINT64); }

inline DATATYPE DataTypeOf (float)
	{ return (DATATYPE_FLOAT32); }

inline DATATYPE DataTypeOf (double)
	{ return (DATATYPE_FLOAT64); }

namespace SYSTEM {

//! Get datatype large enough to hold any value in either specified datatype.
DATATYPE GetDataTypeCombined (
	DATATYPE DataType1,
	DATATYPE DataType2,
	bool IncludeInt64 = false			//!< Return DATATYPE_SINT64 if combine UINT32 and SINT32, otherwise returns DATATYPE_FLOAT64
	);

//! Get maximum valid value for data type.
double GetMaximumValue (
	DATATYPE DataType
	);

//! Get minimum valid value for data type.
double GetMinimumValue (
	DATATYPE DataType
	);

//! Get next largest DATATYPE of same class.
//! Will return DATATYPE_FLOAT64 given largest integer type.
//! Will return DATATYPE_Invalid given largest floating type
DATATYPE GetNextDataType (
	DATATYPE DataType,
	bool IncludeInt64				//!< Include 64-bit integer types
	);

};	// End namespace SYSTEM

};	// End namespace SPATMOD

#endif	 // INC_SPATMOD_DATATYPE_H
