/**
 * \file imagefilteroverlay.h <spatmod/imagefilteroverlay.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_OVERLAY definition.
 *
 * \if NODOC
 * $Id: imagefilteroverlay.h_v 1.3 2010/01/12 20:42:27 bcolombe Exp $
 *
 * $Log: imagefilteroverlay.h_v $
 * Revision 1.3  2010/01/12 20:42:27  bcolombe
 * *** empty log message ***
 *
 * Revision 1.2  2009/08/10 15:54:06  bcolombe
 * Add additional overlap methods
 *
 * Revision 1.1  2007/12/21 22:00:23  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTEROVERLAY_H
#define	INC_SPATMOD_IMAGEFILTEROVERLAY_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;
class SAMPLEITERATOR;
class SAMPLEITERATOR_CONST;

//=====================================================================================================================

//! Filter to overlay multiple source images.
//! The first source image pixel that is valid will used as the output pixel.
//! All sources images must have the same dimensions and pixel type.
class FILTER_OVERLAY : public FILTER {
	public:
		enum OVERLAPMETHOD {
			OVERLAPMETHOD_First,						//!< Use first valid pixel
			OVERLAPMETHOD_Last,						//!< Use last valid pixel
			OVERLAPMETHOD_Minimum,					//!< Use pixel with minimum total of sample values
			OVERLAPMETHOD_Maximum,					//!< Use pixel with maximum total of sample values
			OVERLAPMETHOD_ClosestToSpecified		//!< Use pixel closest to specified value
			};

		//! Construct using array of previous stage pointers and common ownership.
		//! All previous stages must have same total dimensions and pixel type.
		FILTER_OVERLAY (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			OVERLAPMETHOD OverlapMethod	= OVERLAPMETHOD_First //!< Overlap control method
			);

		// Destructor.
		virtual ~FILTER_OVERLAY ();

		//! Set pixel for OVERLAPMETHOD_ClosestToSpecified.
		//! If not specified, the center of the valid data range will be used by default.
		ERRVALUE SetPixelTestClosest (
			const PIXEL& PixelTestClosest			//!< Pixel value, must have same properties as image itself
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FILTER BASECLASS;

		OVERLAPMETHOD m_OverlapMethod;
		PIXEL m_PixelTestClosest;
		double *m_dValueTestClosest;
		SAMPLEITERATOR_CONST *m_itInput;
		SAMPLEITERATOR *m_itOutput;

		void InitPixelTestClosest ();
		ERRVALUE ProcessOverlap (BUFFER& BufferOutput, const BUFFER& BufferInput, INT32 NumColumns, INT32 NumRows) const;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_OVERLAY (const FILTER_OVERLAY&);
		FILTER_OVERLAY& operator= (const FILTER_OVERLAY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTEROVERLAY_H

