/**
 * \file spatmod/imagesampleiterator.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SAMPLEITERATOR class definition.
 *
 * \if NODOC
 * $Id: imagesampleiterator.h_v 1.9 2013/01/02 17:31:19 mju Exp $
 *
 * $Log: imagesampleiterator.h_v $
 * Revision 1.9  2013/01/02 17:31:19  mju
 * Use const for ptr in sampleiterator_const.
 *
 * Revision 1.8  2008/08/28 13:58:52  mju
 * Fix doxygen file ref.
 *
 * Revision 1.7  2007/04/23 15:11:18  mju
 * Add operator += and -=.
 *
 * Revision 1.6  2006/08/03 17:31:16  mju
 * Add putBool and support delegate.
 *
 * Revision 1.5  2006/08/03 13:57:03  mju
 * Add getBool variant with offset.
 *
 * Revision 1.4  2006/08/03 13:51:03  mju
 * Add getBool and supporting delegate.
 *
 * Revision 1.3  2006/07/14 21:49:47  mju
 * Create iterator_const.
 *
 * Revision 1.2  2006/02/10 16:14:17  mju
 * Add getPtr with offset.
 *
 * Revision 1.1  2006/01/30 17:40:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESAMPLEITERATOR_H
#define	INC_SPATMOD_IMAGESAMPLEITERATOR_H

#ifndef	INC_SPATMOD_DATATYPE_H
#include <spatmod/datatype.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class SAMPLEITERATOR;

//=====================================================================================================================

//! Iterator for accessing constant samples in memory array.
class SAMPLEITERATOR_CONST {
	public:

		//! Default constructor.
		SAMPLEITERATOR_CONST ();

		//! Construct with pointer and data type.
		SAMPLEITERATOR_CONST (
			const void *pSample,
			DATATYPE DataType
			);

		//! Destructor.
		~SAMPLEITERATOR_CONST ();

		//! Pre-increment operator.
		SAMPLEITERATOR_CONST& operator++ (
			) { m_pSample += m_SampleBytes; return (*this); }

		//! Pre-decrement operator.
		SAMPLEITERATOR_CONST& operator-- (
			) { m_pSample -= m_SampleBytes; return (*this); }

		//! Increment by value operator.
		SAMPLEITERATOR_CONST& operator+= (
			int num
			) { m_pSample += m_SampleBytes * num; return (*this); }

		//! Decrement by value operator.
		SAMPLEITERATOR_CONST& operator-= (
			int num
			) { m_pSample -= m_SampleBytes * num; return (*this); }

		//! Attach iterator to buffer with data type.
		//! Buffer must remain valid as long as iterator is in use.
		ERRVALUE Attach (
			const void *pSample,				//!< Pointer to sample buffer to attach
			DATATYPE DataType					//!< Sample data type
			);

		//! Attach iterator to buffer, retaining previous data type.
		//! Buffer must remain valid as long as iterator is in use.
		void Attach (
			const void *pSample				//!< Pointer to sample buffer to attach
			) { m_pSample = static_cast<const UINT8*>(pSample); }

		//! Get value pointed to by iterator as bool.
		bool GetBool (
			) const { return (m_DelegateGetBool(m_pSample)); }

		//! Get value pointed to by iterator as bool with offset.
		//! No range checking is done for offset validity.
		bool GetBool (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (m_DelegateGetBool(m_pSample+offset*m_SampleBytes)); }

		//! Get value pointed to by iterator as double.
		double GetDouble (
			) const { return (m_DelegateGetDouble(m_pSample)); }

		//! Get value pointed to by iterator as double with offset.
		//! No range checking is done for offset validity.
		double GetDouble (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (m_DelegateGetDouble(m_pSample+offset*m_SampleBytes)); }

		//! Get multiple values as double-precision.
		void GetDoubles (
			double *values,					//!< Array to hold result
			int count							//!< Number of samples to get
			) const { m_DelegateGetDoubles(m_pSample,values,count); }

		//! Get multiple values as double-precision, with offset.
		void GetDoubles (
			double *values,					//!< Array to hold result
			int count,							//!< Number of samples to get
			int offset							//!< Position before (-) or after (+) current value.
			) const { m_DelegateGetDoubles(m_pSample+offset*m_SampleBytes,values,count); }

		//! Get value pointed to by iterator as INT32, using floor for floating-point conversion.
		INT32 GetINT32_Floor (
			) const { return (static_cast<INT32>(m_DelegateGetUINT32_Floor(m_pSample))); }

		//! Get value pointed to by iterator as INT32 with offset, using floor for floating-point conversion.
		//! No range checking is done for offset validity.
		INT32 GetINT32_Floor (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (static_cast<INT32>(m_DelegateGetUINT32_Floor(m_pSample+offset*m_SampleBytes))); }

		//! Get value pointed to by iterator as UINT32, using floor for floating-point conversion.
		UINT32 GetUINT32_Floor (
			) const { return (m_DelegateGetUINT32_Floor(m_pSample)); }

		//! Get value pointed to by iterator as UINT32 with offset, using floor for floating-point conversion.
		//! No range checking is done for offset validity.
		UINT32 GetUINT32_Floor (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (m_DelegateGetUINT32_Floor(m_pSample+offset*m_SampleBytes)); }

		//! Get pointer to sample.
		const void *GetPtr (
			) const { return (m_pSample); }

		//! Get pointer to sample.
		const void *GetPtr (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (m_pSample+offset*m_SampleBytes); }

		//! Get number of bytes in sample.
		int GetSampleBytes (
			) const { return (m_SampleBytes); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FastDelegate<bool(const void*)> DELEGATE_GETBOOL;
		typedef FastDelegate<double(const void*)> DELEGATE_GETDOUBLE;
		typedef FastDelegate<void(const void*, double*, int)> DELEGATE_GETDOUBLES;
		typedef FastDelegate<UINT32(const void*)> DELEGATE_GETUINT32;
		typedef FastDelegate<void(void*, bool)> DELEGATE_PUTBOOL;
		typedef FastDelegate<void(void*, double)> DELEGATE_PUTDOUBLE;

		const UINT8 *m_pSample;
		int m_SampleBytes;
		DELEGATE_GETBOOL m_DelegateGetBool;
		DELEGATE_GETDOUBLE m_DelegateGetDouble;
		DELEGATE_GETDOUBLES m_DelegateGetDoubles;
		DELEGATE_GETUINT32 m_DelegateGetUINT32_Floor;
		DELEGATE_PUTBOOL m_DelegatePutBool;
		DELEGATE_PUTDOUBLE m_DelegatePutDoubleFastRound;
		DELEGATE_PUTDOUBLE m_DelegatePutDoubleFastFloor;
		DELEGATE_PUTDOUBLE m_DelegatePutDoubleLimitRound;
		DELEGATE_PUTDOUBLE m_DelegatePutDoubleLimitFloor;

		friend class SAMPLEITERATOR;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Iterator for accessing samples in memory array.
class SAMPLEITERATOR : public SAMPLEITERATOR_CONST {
	public:

		//! Default constructor.
		SAMPLEITERATOR (
			) { }

		//! Construct with pointer and data type.
		SAMPLEITERATOR (
			void *pSample,
			DATATYPE DataType
			): SAMPLEITERATOR_CONST(pSample,DataType) { }

		//! Pre-increment operator.
		SAMPLEITERATOR& operator++ (
			) { m_pSample += m_SampleBytes; return (*this); }

		//! Pre-decrement operator.
		SAMPLEITERATOR& operator-- (
			) { m_pSample -= m_SampleBytes; return (*this); }

		//! Get pointer to sample.
		void *GetPtr (
			) const { return (const_cast<UINT8*>(m_pSample)); }

		//! Get pointer to sample.
		void *GetPtr (
			int offset							//!< Position before (-) or after (+) current value.
			) const { return (const_cast<UINT8*>(m_pSample+offset*m_SampleBytes)); }

		//! Put bool value into buffer.
		void PutBool (
			bool value
			) const { m_DelegatePutBool(const_cast<UINT8*>(m_pSample),value); }

		//! Put value into buffer from pointer.
		void PutValueFast (
			const void *pValue				//!< Pointer to source value, must be of proper type and alignment
			) const { if (m_SampleBytes == 1) *const_cast<UINT8*>(m_pSample) = *static_cast<const UINT8*>(pValue); else memcpy(const_cast<UINT8*>(m_pSample),pValue,m_SampleBytes); }

		//! Put value into buffer with no limit validation, rounding if integer buffer.
		void PutValueFastRound (
			double value
			) const { m_DelegatePutDoubleFastRound(const_cast<UINT8*>(m_pSample),value); }

		//! Put value into buffer with no limit validation, using floor if integer buffer.
		void PutValueFastFloor (
			double value
			) const { m_DelegatePutDoubleFastFloor(const_cast<UINT8*>(m_pSample),value); }

		//! Put value into buffer with limit validation, rounding if integer buffer.
		void PutValueLimitRound (
			double value
			) const { m_DelegatePutDoubleLimitRound(const_cast<UINT8*>(m_pSample),value); }

		//! Put value into buffer with limit validation, using floor if integer buffer.
		void PutValueLimitFloor (
			double value
			) const { m_DelegatePutDoubleLimitFloor(const_cast<UINT8*>(m_pSample),value); }

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESAMPLEITERATOR_H
